import numpy as np


class Higham(Exception):
    pass


def hilb(n, m=0):
    """
    hilb   Hilbert matrix.
       hilb(n,m) is the n-by-m matrix with elements 1/(i+j-1).
       it is a famous example of a badly conditioned matrix.
       cond(hilb(n)) grows like exp(3.5*n).
       hilb(n) is symmetric positive definite, totally positive, and a
       Hankel matrix.

       References:
       M.-D. Choi, Tricks or treats with the Hilbert matrix, Amer. Math.
           Monthly, 90 (1983), pp. 301-312.
       N.J. Higham, Accuracy and Stability of Numerical Algorithms,
           Society for Industrial and Applied Mathematics, Philadelphia, PA,
           USA, 2002; sec. 28.1.
       M. Newman and J. Todd, The evaluation of matrix inversion
           programs, J. Soc. Indust. Appl. Math., 6 (1958), pp. 466-476.
       D.E. Knuth, The Art of Computer Programming,
           Volume 1, Fundamental Algorithms, second edition, Addison-Wesley,
           Reading, Massachusetts, 1973, p. 37.

       NOTE added in porting.  We do not use the function cauchy here to
       generate the Hilbert matrix.  That is done so we can unit test the
       the functions against each other.  Also, the function has been
       generalized to take by row and column sizes.  If only a row size
       is given, we assume a square matrix is desired.
    """
    if n < 1 or m < 0:
        raise Higham("Matrix size must be one or greater")
    elif n == 1 and (m == 0 or m == 1):
        return np.array([[1]])
    elif m == 0:
        m = n

    v = np.arange(1, n + 1) + np.arange(0, m)[:, np.newaxis]
    return 1. / v




from rogues.matrices.cauchy import *
from rogues.matrices.chebspec import *
from rogues.matrices.chebvand import *
from rogues.matrices.chow import *
from rogues.matrices.clement import *
from rogues.matrices.comp import *
from rogues.matrices.compan import *
from rogues.matrices.condex import *
from rogues.matrices.cycol import *
from rogues.matrices.dingdong import *
from rogues.matrices.dorr import *
from rogues.matrices.dramadah import *
from rogues.matrices.fiedler import *
from rogues.matrices.forsythe import *
from rogues.matrices.frank import *
from rogues.matrices.gearm import *
from rogues.matrices.gfpp import *
from rogues.matrices.grcar import *
from rogues.matrices.hadamard import *
from rogues.matrices.hankel import *
from rogues.matrices.hanowa import *
from rogues.matrices.hilb import *
from rogues.matrices.invhess import *
from rogues.matrices.invol import *
from rogues.matrices.ipjfact import *
from rogues.matrices.jordbloc import *
from rogues.matrices.kahan import *
from rogues.matrices.kms import *
from rogues.matrices.krylov import *
from rogues.matrices.lauchli import *
from rogues.matrices.lehmer import *
from rogues.matrices.lesp import *
from rogues.matrices.lotkin import *
from rogues.matrices.minij import *
from rogues.matrices.moler import *
from rogues.matrices.neumann import *
from rogues.matrices.ohess import *
from rogues.matrices.parter import *
from rogues.matrices.pascal import *
from rogues.matrices.pdtoep import *
from rogues.matrices.pei import *
from rogues.matrices.pentoep import *
from rogues.matrices.poisson import *
from rogues.matrices.prolate import *
from rogues.matrices.qmult import *
from rogues.matrices.rando import *
from rogues.matrices.randsvd import *
from rogues.matrices.redheff import *
from rogues.matrices.riemann import *
from rogues.matrices.smoke import *
from rogues.matrices.triw import *
from rogues.matrices.wathen import *
from rogues.matrices.wilk import *
from rogues.matrices.wilkinson import *



import numpy as np
import scipy as sp
from rogues.utils import cpltaxes
import matplotlib.pylab as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm


class Higham(Exception):
    pass


def pscont(a, k=0, npts=None, ax=None, levels=None):
    """
    NOTE: Porting to Python/Numpy/Matplotlib has been less than
          ideal.  It currently only 'sorta, kinda works'

    pscont   Contours and colour pictures of pseudospectra.
         pscont(a, k, npts, ax, levels) plots LOG10(1/NORM(R(z))),
         where R(z) = INV(z*I-A) is the resolvent of the square matrix A,
         over an npts-by-npts grid.
         npts defaults to a SIZE(A)-dependent value.
         The limits are ax[0] and ax[1] on the x-axis and
                        ax[2] and ax[3] on the y-axis.
         If ax is omitted, suitable limits are guessed based on the
         eigenvalues of A.
         The eigenvalues of A are plotted as crosses `x'.
         k determines the type of plot:
             k = 0 (default) PCOLOR and CONTOUR
             k = 1           PCOLOR only
             k = 2           SURFC (SURF and CONTOUR)
             k = 3           SURF only
             k = 4           CONTOUR only
         The contours levels are specified by the vector LEVELS, which
         defaults to -10:-1 (recall we are plotting log10 of the data).
         Thus, by default, the contour lines trace out the boundaries of
         the epsilon pseudospectra for epsilon = 1e-10, ..., 1e-1.
         [X, Y, Z, NPTS] = PSCONT(A, ...) returns the plot data X, Y, Z
         and the value of NPTS used.

         After calling this function you may want to change the
         color map (e.g., type COLORMAP HOT - see HELP COLOR) and the
         shading (e.g., type SHADING INTERP - see HELP INTERP).
         For an explanation of the term `pseudospectra', and references,
         see PS.M.
         When A is real and the grid is symmetric about the x-axis, this
         routine exploits symmetry to halve the computational work.

         Colour pseduospectral pictures of this type are referred to as
         `spectral portraits' by Godunov, Kostin, and colleagues.
         References: see PS.
    """

    if np.diff(a.shape)[0] != 0:
        raise Higham('Matrix must be square.')

    n = np.max(a.shape)
    is_a_real = not (a.imag).any()

    if levels is None:
        levels = np.arange(-10, 0)

    e, v = np.linalg.eig(a)

    if ax is None:
        ax = cpltaxes(e, plt)
        if is_a_real:
            ax[2] = -ax[3]

    if npts is None:
        npts = int(3*np.round(min(max(5, np.sqrt(20**2 * 10**3 / n**3)), 30)))

    nptsx = npts
    nptsy = npts
    # ysymmetry = is_a_real and (ax[2] == -ax[3])
    ysymmetry = False  # Hack... to be fixed

    x = np.linspace(ax[0], ax[1], npts)
    y = np.linspace(ax[2], ax[3], npts)
    if ysymmetry:                        # Exploit symmetry about x-axis.
        nptsy = int(np.ceil(npts/2))
        y1 = y
        y = y[0:nptsy]

    xx, yy = np.meshgrid(x, y)
    z = xx + 1j * yy
    eye_n = np.eye(n)
    smin = np.zeros((nptsy, nptsx))

    for j in range(nptsx):
        for i in range(nptsy):
            u, s, v = sp.linalg.svd(z[i, j] * eye_n - a)
            smin[i, j] = np.min(s)

    z = np.log10(smin + np.finfo(float).eps)
    if ysymmetry:
        z = np.vstack((z, z[nptsy - np.remainder(npts, 2)::-1, :]))
        y = y1

    if k == 0 or k == 1:
        plt.pcolor(x, y, z)
    elif k == 2 or k == 3:
        fig = plt.figure()
        ax = fig.gca(projection='3d')
        surf = ax.plot_surface(x, y, z, rstride=1, cstride=1,
                               cmap=cm.coolwarm, linewidth=.2,
                               antialiased=True)

    if k == 0:
        plt.contour(x, y, z, levels)
    elif k == 4:
        plt.contour(x, y, z, levels)

    if k != 2 and k != 3:
        if k == 0 or k == 1:
            s = 'w'   # White
        else:
            s = 'k'   # Black
        plt.plot(e.real, e.imag, ''.join((s, 'x')))

    plt.axis('equal')
    plt.show()
    return x, y, z



import numpy as np


def augment(a, alpha=1.):
    """
    AUGMENT  Augmented system matrix.
         AUGMENT(A, ALPHA) is the square matrix
         [alpha*eye(m) a; a' zeros(n)] of dimension m+n, where a is m-by-n.
         It is the symmetric and indefinite coefficient matrix of the
         augmented system associated with a least squares problem
         minimize NORM(A*x-b).  alpha defaults to 1.
         Special case: if "a" is a scalar, n say, then augment(a) is the
                       same as AUGMENT(RANDN(p,q)) where n = p+q and
                       p = ROUND(n/2), that is, a random augmented matrix
                       of dimension n is produced.
         The eigenvalues of AUGMENT(A) are given in terms of the singular
         values s(i) of A (where m>n) by
                  1/2 +/- SQRT( s(i)^2 + 1/4 ),  i=1:n  (2n eigenvalues),
                  1,  (m-n eigenvalues).
         If m < n then the first expression provides 2m eigenvalues and the
         remaining n-m eigenvalues are zero.

         See also SPAUGMENT.

         Reference:
         G.H. Golub and C.F. Van Loan, Matrix Computations, Second
         Edition, Johns Hopkins University Press, Baltimore, Maryland,
         1989, sec. 5.6.4.
    """

    try:
        m, n = a.shape
    except AttributeError:
        # Handle the special case a = scalar
        n = a
        p = np.round(n / 2)
        q = n - p
        a = np.random.randn(p, q)
        m = p
        n = q

    c = np.vstack((np.hstack((alpha * np.eye(m), a)),
                   np.hstack((a.T, np.zeros((n, n))))))

    return c



from scipy.sparse.linalg import bicg
import numpy as np
import numpy.linalg as nl
import numpy.testing as npt
import rogues


def assert_tridiag(a):
    """Return true if input array is tridiagonal"""
    # input arrays must be square
    try:
        if len(a.shape) != 2:
            return False
        m, n = a.shape
        if m != n:
            assert(False)
    except AttributeError:
        assert(False)

    tri = np.tril(np.triu(a, -1), 1)
    npt.assert_array_almost_equal(tri, a)


def test_cauchy():
    """Simple test of cauchy.  We test by checking against a hilbert
    matrix
    """
    x = np.arange(1, 11) - 0.5
    a = rogues.cauchy(x)
    b = rogues.hilb(10)
    npt.assert_array_equal(a, b)

    # scalar argument
    a = rogues.cauchy(10)
    b = rogues.cauchy(np.arange(1, 10))
    npt.assert_array_equal(a, b)

def cheb(xx):
    """Helper function: generator for Chebyshev polynomials"""
    x = xx
    a, b = 1, x
    yield 1.
    while True:
        yield b
        a, b = b, 2. * x * b - a


def test_chebspec():
    """Simple test of chebspec."""
    n = 19
    a = rogues.chebspec(n, k=1)
    w, v = nl.eig(a)
    assert((w.real < 0).all())


def test_chebvand():
    """Simple test of chebvand.  We test by checking against a chebyshev
    polynomial calculated by the test helper function cheb.
    """
    x = 0.8 * np.ones(8)
    a = rogues.chebvand(x)
    b = cheb(0.8)
    c = np.array([next(b) for i in range(8)])
    npt.assert_array_equal(a[:, 0], c)


def test_chow():
    """Simple test of chow.  We test by checking the only the number
    of zero eigenvalues.
    """
    n = 9
    a = rogues.chow(n)
    p = int(np.floor(n / 2))
    num_zero_eigvals = 0
    w, v = nl.eig(a)
    # sad, sad convergence here...
    num_zero_eigvals = (np.abs(w) < 1e-3).sum()
    assert(p == num_zero_eigvals)


def test_clement_a():
    """Simple Test of clement"""
    a = rogues.clement(9)
    w, v = nl.eig(a)
    npt.assert_almost_equal(w.sum(), 0.0, 10)


def test_clement_b():
    """Simple Test of clement, symmetric case"""
    a = rogues.clement(9, k=1)
    w, v = nl.eig(a)
    npt.assert_almost_equal(w.sum(), 0.0, 10)


def test_comp():
    """Simple Test of comp"""
    a  = np.outer(np.arange(1, 7), np.ones(6))
    c  = rogues.comp(a)
    ck = np.array([[1, -1, -1, -1, -1, -1],
                   [-2, 2, -2, -2, -2, -2],
                   [-3, -3, 3, -3, -3, -3],
                   [-4, -4, -4, 4, -4, -4],
                   [-5, -5, -5, -5, 5, -5],
                   [-6, -6, -6, -6, -6, 6]])
    npt.assert_array_equal(c, ck)


def test_compan():
    """Simple Test of compan"""
    a  = np.array([1, 0, -7, 6])
    c  = rogues.compan(a)
    ck = np.array([[0, 7, -6],
                   [1, 0, 0],
                   [0, 1, 0]])
    npt.assert_array_equal(c, ck)


def test_condex():
    """Simple test of condex.  Only tests k=1, should test others"""
    a = np.array([[1, -1, -200, 0],
                  [0, 1, 100, -100],
                  [0, 1, 101, -101],
                  [0, 0, 0, 100]])
    b = rogues.condex(4, 1)
    npt.assert_array_equal(a, b)


def test_cycol():
    """Simple test of cycol(n) for n = 5."""
    a = rogues.cycol(5)
    # Every column should be the same
    result = True
    for i in range(1, 5):
        result = result and (a[:, i] == a[:, 0]).all()
    assert(result)


def test_dingdong():
    """Simple test of dingdong(n) for n = 9. Only tests to
    see if matrix is symmetric. Again, very lame testing,
    but it is a start"""
    a = rogues.dingdong(9)
    # Matrix must be Hermetian
    assert((np.triu(a) == np.triu(np.conj(a.T))).all())


def test_dorr():
    """Simple test of dorr(10).  We actually only check to
    see if the eigenvalue condition numbers are big"""
    a = rogues.dorr(10, r_matrix=True)
    w, v, cond = rogues.condeig(a)
    assert((cond.sum() > 1e15))


def test_dramadah_a():
    """Simple test of dramadah(10,1).  We check to that abs(det()) == 1"""
    a = rogues.dramadah(10)
    npt.assert_almost_equal(abs(nl.det(a)), 1, 12)


def test_dramadah_b():
    """Simple test of dramadah(10, 2).  We check that all eigenvalues == 1"""
    a = rogues.dramadah(10, 2)
    w, v = nl.eig(a)
    npt.assert_array_equal(w, np.ones(10))


def fib():
    """Helper function: generator for Fibonnaci numbers"""
    a, b = 0, 1
    while True:
        yield b
        a, b = b, a + b


def test_dramadah_c():
    """Simple test of dramadah(10, 3).  We check that it's determinant is
    the n-th Fibonacci number"""
    a = rogues.dramadah(10, 3)
    x = nl.det(a)
    gf = fib()
    y = [next(gf) for i in range(10)]
    npt.assert_almost_equal(x, y[9])


def test_fiedler():
    """Simple test of fiedler(n) for n = 3. Taken from Higham &
       Higham p. 135
    """
    a = np.array([[0, 1, 2], [1, 0, 1], [2, 1, 0]])
    b = rogues.fiedler(3)
    npt.assert_array_equal(a, b)


def test_forsythe():
    """Simple test of forsythe(10, 3.4, .2)"""
    a = rogues.forsythe(10, .1, 9)
    w, v = nl.eig(a)
    npt.assert_almost_equal(np.abs(w.sum()), 90, 12)


def test_frank():
    """Simple exercise of the function frank(n)"""
    a = rogues.frank(5)
    npt.assert_almost_equal(1., np.linalg.det(a), 5)


def test_gearm():
    """Simple test of gearm(10)"""
    a = rogues.gearm(10)
    w, v = nl.eig(a)
    npt.assert_almost_equal(nl.norm(w), 4., 14)


def test_gfpp():
    """Simple exercise of generating the gfpp matrix
    we know this matrix has a growth factor given by 2**(n - 1), which
    for n = 5 is 16, so we just check for that (assumes ge is correct)
    """
    a = rogues.gfpp(5)
    l, u, r = rogues.ge(a)
    npt.assert_almost_equal(r, 16., 8)


def test_grcar():
    """Simple test of grcar"""
    n = 28
    a = rogues.grcar(n)
    w, v = nl.eig(a)
    npt.assert_almost_equal(w.sum(), n, 12)


def test_hadamard():
    """Simple test of hadamard(16)"""
    n = 16
    a = rogues.hadamard(n)
    dm = a.T @ a
    npt.assert_array_equal(np.diag(dm), n * np.ones(n))


def test_hankel():
    """Simple test of Hankel matrix"""
    a = np.arange(10)
    b = np.arange(9, 19)
    h = rogues.hankel(a, b)

    t = np.array([[ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9],
                  [ 1,  2,  3,  4,  5,  6,  7,  8,  9, 10],
                  [ 2,  3,  4,  5,  6,  7,  8,  9, 10, 11],
                  [ 3,  4,  5,  6,  7,  8,  9, 10, 11, 12],
                  [ 4,  5,  6,  7,  8,  9, 10, 11, 12, 13],
                  [ 5,  6,  7,  8,  9, 10, 11, 12, 13, 14],
                  [ 6,  7,  8,  9, 10, 11, 12, 13, 14, 15],
                  [ 7,  8,  9, 10, 11, 12, 13, 14, 15, 16],
                  [ 8,  9, 10, 11, 12, 13, 14, 15, 16, 17],
                  [ 9, 10, 11, 12, 13, 14, 15, 16, 17, 18]])

    npt.assert_array_equal(h, t)


def test_hanowa():
    """Simple test of hanowa matrices. Real parts of eigenvalue
       must all be one
    """
    n = 10
    a = rogues.hanowa(n)
    w, v = nl.eig(a)
    npt.assert_array_equal(w.real, np.ones(n))


def test_hilb():
    """Simple test of hilb() Hilbert matrices"""
    a = rogues.hilb(10)
    cond = nl.cond(a)
    assert(cond > 1e+12)


def test_invhess():
    """
    Simple test of invhess matrix
    This is really not a very good test, for I don't know of
    any special properties to look for.  However, it looks to
    be very poorly conditioned, so we will just look for that.
    """
    a = rogues.invhess(55)
    w, v, cond = rogues.condeig(a)
    assert((cond > 1e+14).any())


def test_invol():
    """Simple test of invol.  Must be it's own inverse."""
    n = 5
    a = rogues.invol(n)
    b = a @ a
    bd = np.diag(np.ones(n))
    print ("invol**2 = ", b)
    npt.assert_array_almost_equal(b, bd)


def test_ipjfact():
    """Simple test of ipjfact. Use simple minded version"""
    fa = np.arange(200.)
    fa[0] = 1.
    fac = np.cumprod(fa)
    n = 6
    a = np.empty((n, n))
    for i in range(n):
        for j in range(n):
            a[i, j] = fac[i + j + 2]
    b = nl.det(a)
    c, d = rogues.ipjfact(n)
    assert((a == c).all() and ((np.abs(b - d) / np.abs(b)) < 1e-10))


def test_jordbloc():
    """Simple test of jordbloc. Do we recover the correct eigenvalues?"""
    evalue = 5.911  # arbitrary value
    a = rogues.jordbloc(10, evalue)
    w, v = nl.eig(a)
    b = evalue * np.ones(10)
    npt.assert_array_equal(w, b)


def test_kahan():
    """Simple test of kahan matrices.
       In fact, it is very cheesey.  I just check against diag element
       generated by NIST's matrixmarket deli for the kahan matrix
    """
    nist_diag = np.array([1.0000000000000555, \
                          0.9320390859672762, \
                          0.8686968577706671, \
                          0.8096594252991716, \
                          0.7546342307005867, \
                          0.7033485986217525, \
                          0.6555483849757441, \
                          0.6109967175400799, \
                          0.5694728221450271, \
                          0.5307709286352231])
    a = rogues.kahan(10)
    print (np.diag(a))
    print (nist_diag)
    npt.assert_array_almost_equal(np.diag(a), nist_diag)


def test_kms():
    """Test of kms matrix. Tests against a known L*D*L.T factorization"""
    n = 4
    rho = 0.5
    a = rogues.kms(n, rho)

    #  The following is an advertised property of the kms matrix, but I
    #  have not been able to verify.  The kms matrices are very simple, and
    #  easy to verify by inspection so the problem is either in the stated
    #  property or in my implementation of it.
    l = nl.inv(rogues.triw(n, -rho, 1).T)
    d = (1 - np.abs(rho) ** 2) * np.eye(n)
    d[0, 0] = 1
    k = l @ d @ l.T
    npt.assert_array_equal(a, k)

    # verify that nl.inv(a) is tridiagonal.
    b = nl.inv(a)
    assert_tridiag(b)


def test_krylov():
    """Simple test of krylov matrix"""
    d = np.array([[1, 10, 40, 160],
                  [2, 10, 40, 160],
                  [3, 10, 40, 160],
                  [4, 10, 40, 160]])
    a = np.ones((4, 4))
    b = np.arange(1, 5)
    c = rogues.krylov(a, b)
    npt.assert_array_equal(c, d)


def test_lauchli():
    """Simple test of the lauchli matrix."""
    a = np.array([[1, 1, 1, 1], [5, 0, 0, 0], [0, 5, 0, 0], [0, 0, 5, 0],\
                  [0, 0, 0, 5]])
    b = rogues.lauchli(4, 5)
    npt.assert_array_equal(a, b)


def test_lehmer():
    """Simple test of lehmer matrix."""
    a = np.array([[1 / 1., 1 / 2., 1 / 3., 1 / 4.],
                  [1 / 2., 2 / 2., 2 / 3., 2 / 4.],
                  [1 / 3., 2 / 3., 3 / 3., 3 / 4.],
                  [1 / 4., 2 / 4., 3 / 4., 4 / 4.]])
    b = rogues.lehmer(4)
    npt.assert_array_equal(a, b)


def test_lesp():
    """Simple test of lesp matrix"""
    n = 5
    b = rogues.lesp(n)
    c = np.diag(b, -1)
    d = 1 / np.arange(2., n + 1)
    assert_tridiag(b)
    npt.assert_array_equal(c, d)


def test_lotkin():
    """Simple test of lotkin matrix."""
    a = rogues.lotkin(10)
    # supposed to have many negative eigenvalues of small magnitude
    w, v = nl.eig(a)
    assert((w < 0).sum() == 9  and  (abs(w) < 1e-5).sum() == 5)


def test_minij():
    """Simple test of minij matrix."""
    n = 10
    a = rogues.minij(n)
    b = nl.inv(a)
    assert_tridiag(b)
    assert(b[n - 1, n - 1] == 1)


def test_moler():
    """Simple test of moler(n) for n = 3. Taken from Higham & Higham p. 135"""
    a = np.array([[1, -1, -1], [-1, 2, 0], [-1, 0, 3]])
    b = rogues.moler(3)
    npt.assert_array_equal(a, b)


def test_neumann():
    """Simple test of neumann matrix."""
    # OK, this is insane, but these _seem_ to be the eigenvalues...
    ans = np.array([0., 1, 1, 2, 3, 3, 4, 4, 4, 4, 5, 5, 6, 7, 7, 8])
    a, t = rogues.neumann(16)
    w, v = nl.eig(a)
    x = w.real
    #
    # for some reason, cannot put the x.sort() directly into the allclose func
    # and must do it first...
    x.sort()
    assert(np.allclose(x, ans))


def test_ohess():
    """Simple test of ohess matrix."""
    n = 10
    a = rogues.ohess(n)
    # Test to see if a is orthogonal...
    b = a @ a.T
    assert(np.allclose(b, np.eye(n)))

# def test_othog():
#     """Simple test of orthog matrix."""
#     # We actually need to test all the options, but...
#     n = 10
#     a = rogues.orthog(n)
#     # Test to see if a is orthogonal...
#     b = a @ a.T
#     assert( np.allclose(b, np.eye(n)) )


def test_parter():
    """Simple test of parter matrix."""
    # Well, this is not a real test, only a twiddle
    n = 10
    a = rogues.parter(n)
    npt.assert_almost_equal(np.abs(nl.det(a)), 33961.9329448)


def test_pascal_0():
    """Simple test of pascal matrix: k = 0."""
    # Note that this test will fail if n > 12
    n = 12
    a = rogues.pascal(n)
    w, v = nl.eig(a)
    b = w * w[::-1]
    npt.assert_array_almost_equal(b, np.ones(n))


def test_pascal_1():
    """Simple test of pascal matrix: k = 1."""
    # Notice we recover the unit matrix with n = 18, better than previous test
    n = 18
    a = rogues.pascal(n, 1)
    b = a @ a
    assert(np.allclose(b, np.eye(n)))


def test_pdtoep():
    """Simple test of pdtoep matrix."""
    # just test positive definite at some random vector
    n = 10
    a = rogues.pdtoep(n)
    x = np.random.rand(n)
    b = np.dot(x, np.dot(x, a))
    assert(b > 0)


def test_pei():
    """Simple test of pei matrix."""
    n = 10
    a = rogues.pei(n)
    npt.assert_almost_equal(nl.det(a), 11.0)


def test_pentoep():
    """Simple test of pentoep matrix."""
    # incredibly lame, essentially just test that we don't assert
    n = 18
    a = rogues.pentoep(n)
    assert(np.abs(nl.det(a)) > 1.0)


def test_poisson():
    """Simple test of poisson matrix."""
    # incredibly lame, essentially just test that we don't assert
    n = 10
    a = rogues.poisson(n)
    assert(a.shape[0] == 100)


def test_prolate():
    """Simple test of prolate matrix."""
    n = 18
    a = rogues.prolate(n)
    w, v = nl.eig(a)
    assert((w > 0).all() and (w < 1).all())


def test_qmult():
    """Simple test of qmult."""
    n = 18
    a = rogues.qmult(n)
    npt.assert_almost_equal(np.abs(nl.det(a)), 1.0)


def test_rando():
    """Simple test of rando matrix generation."""
    n = 18
    a = rogues.rando(n, k=2)
    assert((np.abs(a)).sum() == n ** 2)


def test_randsvd():
    """Simple test of randsvd THIS WILL FAIL ~10% of the time"""
    n = 18
    k = 1e+20
    a = rogues.randsvd(n, kappa=k)
    c = nl.cond(a) / k   # so c _should_ be close to one (yeah, right)

    # The actual condition number should be within an order of magnitude
    # of what we asked for but, sometimes even a order of magnitude doesn't
    # quite cut it so we loosen up even more (even this doesn't always pass
    # so we must think of a better check (looks like this fails a little
    # less than 10% of the time.
    #print ('c = %f' % c)
    #
    assert(c > 0.001 and c < 1000.)


def test_redheff():
    """Simple test of Redheffer matrix."""
    n = 20
    num_one_eigs = n - np.floor(np.log2(n)) - 1
    a = rogues.redheff(n)
    w, v = nl.eig(a)
    count = 0
    for x in w:
        if round(x.real, 3) == 1:
            count += 1

    npt.assert_equal(num_one_eigs, count)


def test_riemann():
    """Simple test of the Riemann matrix."""
    n = 10
    a = rogues.riemann(n)
    b = np.tril(-np.ones((n, n)), -1)
    c = np.tril(a, -1)
    # Kind of a goofy prop to check, but it's simple
    npt.assert_array_equal(b, c)


def test_smoke():
    """Simple test of the smoke matrix."""
    n = 32
    a = rogues.smoke(n)
    w, v = nl.eig(a)
    assert(
        np.allclose(np.sqrt((w * np.conj(w)).real), 2 ** \
                              (1 / float(n)) * np.ones(32)))


def test_triw():
    """Simple test of triw(n) for n = 5"""
    a = np.array([[1, -1, -1, -1, -1],
                  [0, 1, -1, -1, -1],
                  [0, 0, 1, -1, -1],
                  [0, 0, 0, 1, -1],
                  [0, 0, 0, 0, 1]])
    b = rogues.triw(5)
    npt.assert_array_equal(a, b)


def test_wathen():
    """Simple test of wathen(n,n) for n = 5"""
    a = rogues.wathen(5, 5)
    n = a.shape[0]
    b = np.ones(n)
    x, info = bicg(a, b)
    npt.assert_equal(info, 0)
    # since wathen is symmetric, positive definite,
    # it had better have all real eigs
    npt.assert_almost_equal((x.imag).sum(), 0.0)


def test_wilk():
    """Simple test of wilk(4)"""
    a, b = rogues.wilk(4)
    w = np. array([[0.9143e-4, 0, 0, 0],
                   [0.8762, 0.7156e-4, 0, 0],
                   [0.7943, 0.8143, 0.9504e-4, 0],
                   [0.8017, 0.6123, 0.7165, 0.7123e-4]])
    npt.assert_array_equal(a, w)


def test_wilkinson():
    """Simple test of wilkinson(21)"""
    a, b = rogues.wilk(21)
    w = rogues.wilkinson(21)
    npt.assert_array_equal(a, w)


if __name__ == "__main__":
    npt.run_module_suite()

#IgnoreException
#NumpyTest
#NumpyTestCase
#ParametricTestCase
#TestCase
#Tester
#__builtins__
#__doc__
#__file__
#__name__
#__path__
#assert_almost_equal
#assert_approx_equal
#assert_array_almost_equal
#assert_array_equal
#assert_array_less
#assert_equal
#assert_raises
#assert_string_equal
#build_err_msg
#dec
#decorate_methods
#decorators
#importall
#jiffies
#measure
#memusage
#nosetester
#numpytest
#parametric
#print_assert_equal
#raises
#rand
#restore_path
#run_module_suite
#rundocs
#runstring
#set_local_path
#set_package_path
#test
#utils
#verbose



import numpy as np


def cauchy(x, y=None):
    """
    cauchy  cauchy matrix.
        c = cauchy(x, y), where x, y are n-vectors, is the n-by-n matrix
        with c(i,j) = 1/(x(i)+y(j)).   By default, y = x.
        Special case: if x is a scalar cauchy(x) is the same as cauchy(1:x).
        Explicit formulas are known for DET(C) (which is nonzero if X and Y
        both have distinct elements) and the elements of INV(C).
        C is totally positive if 0 < X(1) < ... < X(N) and
        0 < Y(1) < ... < Y(N).

        References:
        N.J. Higham, Accuracy and Stability of Numerical Algorithms,
           Society for Industrial and Applied Mathematics, Philadelphia, PA,
           USA, 2002; sec. 28.1.
        D.E. Knuth, The Art of Computer Programming, Volume 1,
           Fundamental Algorithms, second edition, Addison-Wesley, Reading,
           Massachusetts, 1973, p. 36.
        E.E. Tyrtyshnikov, Cauchy-Toeplitz matrices and some applications,
           Linear Algebra and Appl., 149 (1991), pp. 1-18.
        O. Taussky and M. Marcus, Eigenvalues of finite matrices, in
           Survey of Numerical Analysis, J. Todd, ed., McGraw-Hill, New York,
           pp. 279-313, 1962. (States the totally positive property on p. 295.)
    """

    if not hasattr(x, 'shape'):
        x = np.arange(1, x)

    if y is None:
        y = x

    if not x.shape == y.shape:
        raise ValueError('Parameter vectors must be of same dimension.')

    n = x.shape[0]
    c = np.outer(x, np.ones(n)) + np.outer(np.ones(n), y)
    return 1. / c



import numpy as np
from rogues.matrices.triw import triw


def moler(n, alpha=-1):
    """
    MOLER   Moler matrix - symmetric positive definite.
        A = MOLER(N, ALPHA) is the symmetric positive definite N-by-N matrix
        U'*U where U = TRIW(N, ALPHA).
        For ALPHA = -1 (the default) A(i,j) = MIN(i,j)-2, A(i,i) = i.
        A has one small eigenvalue.

        Nash (1990) attributes the ALPHA = -1 matrix to Moler.

        Reference:
        J.C. Nash, Compact Numerical Methods for Computers: Linear
        Algebra and Function Minimisation, second edition, Adam Hilger,
        Bristol, 1990 (Appendix 1).
    """
    a = triw(n, alpha).T  @  triw(n, alpha)
    return a



import numpy as np
import numpy.random as nrnd


def krylov(a, x=None, j=None):
    """
    KRYLOV    Krylov matrix.
          krylov(a, x, j) is the Krylov matrix
          [x, ax, a^2x, ..., a^(j-1)x],
          where a is an n-by-n matrix and x is an n-vector.
          defaults: x = np.ones(n), j = n.
          krylov(n) is the same as krylov(randn(n)).

          Reference:
          G.H. Golub and C.F. Van Loan, Matrix Computations, second edition,
          Johns Hopkins University Press, Baltimore, Maryland, 1989, p. 369.
    """
    try:
        n, n = a.shape
    except AttributeError:
        n = a
        a = nrnd.randn(n, n)

    if j is None:
        j = n

    if x is None:
        x = np.ones(n)

    b = np.ones((n, j))
    b[:, 0] = x

    for i in range(1, j):
        b[:, i] = np.dot(a, b[:, i - 1])

    return b



import numpy as np


class Higham(Exception):
    pass


def mgs(a):
    """
    MGS     Modified Gram-Schmidt QR factorization.
        q, r = mgs(a) uses the modified Gram-Schmidt method to compute the
        factorization a = q*e for m-by-n a of full rank,
        where q is m-by-n with orthonormal columns and R is n-by-n.
    """
    try:
        m, n = a.shape
    except AttributeError:
        raise Higham("Input array must be two dimensional")

    q = np.zeros((m, n))
    r = np.zeros((n, n))

    for k in range(n):
        r[k, k] = np.linalg.norm(a[:, k])
        q[:, k] = a[:, k] / r[k, k]
        r[k, k + 1:n] = np.dot(q[:, k], a[:, k + 1:n])
        a[:, k + 1:n] = a[:, k + 1:n] - np.outer(q[:, k], r[k, k + 1:n])

    return q, r



import numpy as np
import scipy.sparse as sparse


class Higham(Exception):
    pass


def tridiag(n, x=None, y=None, z=None):
    """
    tridiag  tridiagonal matrix (sparse).
         tridiag(x, y, z) is the sparse tridiagonal matrix with
         subdiagonal x, diagonal y, and superdiagonal z.
         x and z must be vectors of dimension one less than y.
         Alternatively tridiag(n, c, d, e), where c, d, and e are all
         scalars, yields the toeplitz tridiagonal matrix of order n
         with subdiagonal elements c, diagonal elements d, and superdiagonal
         elements e.   This matrix has eigenvalues (todd 1977)
                  d + 2*sqrt(c*e)*cos(k*pi/(n+1)), k=1:n.
         tridiag(n) is the same as tridiag(n,-1,2,-1), which is
         a symmetric positive definite m-matrix (the negative of the
         second difference matrix).

         References:
         J. Todd, Basic Numerical Mathematics, Vol. 2: Numerical Algebra,
           Birkhauser, Basel, and Academic Press, New York, 1977, p. 155.
         D.E. Rutherford, Some continuant determinants arising in physics and
           chemistry---II, Proc. Royal Soc. Edin., 63, A (1952), pp. 232-241.
    """
    try:
        # First see if they are arrays
        nx, = n.shape
        ny, = x.shape
        nz, = y.shape
        if (ny - nx - 1) != 0 or (ny - nz - 1) != 0:
            raise Higham('Dimensions of vector arguments are incorrect.')
        # Now swap to match above
        z = y
        y = x
        x = n

    except AttributeError:
        # They are not arrays
        if n < 2:
            raise Higham("n must be 2 or greater")

        if x is None and y is None and z is None:
            x = -1
            y = 2
            z = -1

        x = x * np.ones(n - 1)
        z = z * np.ones(n - 1)
        y = y * np.ones(n)

    except ValueError:
        raise Higham("x, y, z must be all scalars or 1-D vectors")

    # t = diag(x, -1) + diag(y) + diag(z, 1);  % For non-sparse matrix.
    n = np.max(np.size(y))
    za = np.zeros(1)

    # Use the (*)stack functions instead of the r_[] notation in
    # an attempt to be more readable. (Doesn't look like it helped much)
    t = sparse.spdiags(np.vstack((np.hstack((x, za)), y,
                                  np.hstack((za, z)))),
                       np.array([-1, 0, 1]), n, n)

    return t



import numpy as np
import numpy.testing as npt
import rogues


def rosen(x, *varargs):
    """
    Here we actually compute the negative of the Rosenbrock function
    since mdsmax attempts to find the maximum.
    """
    sum = 0.
    for i in range(x.size - 1):
        sum += (1 - x[i]) ** 2 + 100 * ((x[i + 1] - x[i] ** 2) ** 2)

    return -sum


def exp_func(x, *varargs):
    """
    Very simple function to maximize.  sum over i of exp(-x[i]**2)
    """
    sum = 0.
    for z in x:
        sum += np.exp(-z ** 2)

    return sum


def test_mds_exp():
    n = 5
    z = np.ones(n)
    x, fmax, nf = rogues.mdsmax(exp_func, z)
    va = []
    npt.assert_almost_equal(exp_func(np.zeros(n), va), exp_func(x, va))


def test_mds_rosen():
    # so the starting point is kind of cooked...
    n = 5
    z = 0.9 * np.ones(n)
    stopit = (1e-12, np.inf, np.inf, 0, 1)
    x, fmax, nf = rogues.mdsmax(rosen, z, stopit)
    npt.assert_almost_equal(np.ones(n), x)


if __name__ == "__main__":
    npt.run_module_suite()



import numpy as np


def fiedler(c):
    """
    FIEDLER  Fiedler matrix - symmetric.
         A = FIEDLER(C), where C is an n-vector, is the n-by-n symmetric
         matrix with elements ABS(C(i)-C(j)).
         Special case: if C is a scalar, then A = FIEDLER(1:C)
                       (i.e. A(i,j) = ABS(i-j)).
         Properties:
           FIEDLER(N) has a dominant positive eigenvalue and all the other
                      eigenvalues are negative (Szego, 1936).
           Explicit formulas for INV(A) and DET(A) are given by Todd (1977)
           and attributed to Fiedler.  These indicate that INV(A) is
           tridiagonal except for nonzero (1,n) and (n,1) elements.
           [I think these formulas are valid only if the elements of
           C are in increasing or decreasing order---NJH.]

           References:
           G. Szego, Solution to problem 3705, Amer. Math. Monthly,
              43 (1936), pp. 246-259.
           J. Todd, Basic Numerical Mathematics, Vol. 2: Numerical Algebra,
              Birkhauser, Basel, and Academic Press, New York, 1977, p. 159.
    """

    try:
        if len(c.shape) != 1:
            raise ValueError("Only 1-D vectors or scalers are valid input")
        n, = c.shape

    except AttributeError:
        # c must be a scalar integer
        n = c
        c = np.arange(1, n + 1)

    a = np.ones((n, n)) * c
    a = abs(a - a.T)        # NB. array transpose.

    return a



import numpy as np


class Higham(Exception):
    pass


def ohess(x):
    """
    ohess  random, orthogonal upper hessenberg matrix.
       h = ohess(n) is an n-by-n real, random, orthogonal
       upper Hessenberg matrix.
       Alternatively, H = OHESS(X), where X is an arbitrary real
       N-vector (N > 1) constructs H non-randomly using the elements
       of X as parameters.
       In both cases H is constructed via a product of N-1 Givens rotations.

       Note: See Gragg (1986) for how to represent an N-by-N (complex)
       unitary Hessenberg matrix with positive subdiagonal elements in terms
       of 2N-1 real parameters (the Schur parameterization).
       This py-file handles the real case only and is intended simply as a
       convenient way to generate random or non-random orthogonal Hessenberg
       matrices.

       Reference:
       W.B. Gragg, The QR algorithm for unitary Hessenberg matrices,
       J. Comp. Appl. Math., 16 (1986), pp. 1-8.
    """

    if type(x) == int:
        n = x
        x = np.random.uniform(size=n - 1) * 2 * np.pi
        h = np.eye(n)
        h[n - 1, n - 1] = np.sign(np.random.randn())
    elif type(x) == numpy.ndarray:
        if np.imag(x).any():
            raise Higham('Parameter must be real.')
        n = np.max(x.shape)
        h = np.eye(n)
        # Second term ensures h[n-1, n-1] nonzero.
        h[n - 1, n - 1] = np.sign(x[n - 1]) + float(x[n - 1] == 0)
    else:
        raise Higham('Unknown type in ohess')        
        
        
    for i in range(n - 1, 0, -1):
        # Apply Givens rotation through angle x[i - 1]
        theta = x[i - 1]
        c = np.cos(theta)
        s = np.sin(theta)
        h[i - 1:i + 1, :] = np.vstack((c * h[i - 1, :] + s * h[i, :],
                                       -s * h[i - 1, :] + c * h[i, :]))

    return h



import numpy as np


class Higham(Exception):
    pass


def toeplitz(a, b=None):
    """
    toeplitz(a) returns a toeplitz matrix given "a", the first row of the
    matrix.  This matrix is defined as:

        [ [   a[0], a[1], a[2], a[3], ...,   a[n] ],
          [   a[n], a[0], a[1], a[2], ..., a[n-1] ],
          [ a[n-1], a[n], a[0], a[1], ..., a[n-2] ],
          ...
          [   a[1], a[2], a[3], a[4], ...,   a[0] ]]

    Note that this array is more properly called a "circulant" because each
    row is a circular shift of the one above it. Also, note that the main
    diagonal is constant and given by a[0].  If the array "a" is complex,
    the elements rotated to be under the main diagonal are complex conjugated
    to yield a Hermetian matrix.

    If called as toeplitz(a, b) then create the unsymetric toeplitz matrix.
    Here "b" would be the first row and "a" would be the first column. (This
    is bass-ackwards to follow the m*lab convention.) The second row shifts
    the first row to the right by one, but instead circularly shifting the
    last element of the row back into the first element, we shift in the
    successive elements of a as the element of b are shifted out.

    See the wikipedia entry for more information and references (especially the
    following: http://ee.stanford.edu/~gray/toeplitz.pdf)
    """

    # Error checking...
    try:
        m, = a.shape
        if b is not None:
            n, = b.shape
    except (ValueError, AttributeError):
        raise Higham("Input arrays must be one dimensional")

    if b is not None:
        if m != n:
            raise Higham("Input arrays must be have the same dimension")
    else:
        # If only one array specified and it is complex, make sure that the
        # generated toeplitz array is Hermitian
        b = np.conj(a.copy())

    if a[0] != b[0]:
        print("Warning: a[0] != b[0]. First vector (toeplitz column) wins")

    t = b.copy()
    t[0] = a[0]
    for i in range(1, m):
        rot = np.hstack((a[i::-1], b[1:-i]))
        t = np.vstack((t, rot))

    return t



import numpy as np


def chebspec(n, k=0):
    """"
    CHEBSPEC  Chebyshev spectral differentiation matrix.
          c = chebspec(n, k) is a Chebyshev spectral differentiation
          matrix of order n.  k = 0 (the default) or 1.
          For k = 0 (`no boundary conditions'), c is nilpotent, with
              c**n = 0 and it has the null vector ones(n,1).
              c is similar to a Jordan block of size n with eigenvalue zero.
              Default is k = 0
          For k = 1, c is nonsingular and well-conditioned, and its eigenvalues
              have negative real parts.
          For both k, the computed eigenvector matrix x from eig is
              ill-conditioned (mesh(real(x)) is interesting).

          References:
          C. Canuto, M.Y. Hussaini, A. Quarteroni and T.A. Zang, Spectral
             Methods in Fluid Dynamics, Springer-Verlag, Berlin, 1988; p. 69.
          L.N. Trefethen and M.R. Trummer, An instability phenomenon in
             spectral methods, SIAM J. Numer. Anal., 24 (1987), pp. 1008-1023.
          D. Funaro, Computing the inverse of the Chebyshev collocation
             derivative, SIAM J. Sci. Stat. Comput., 9 (1988), pp. 1050-1057.
    """

    # k = 1 case obtained from k = 0 case with one bigger n.
    if k == 1:
        n += 1

    n = n - 1
    c = np.zeros((n + 1, n + 1))

    one = np.ones(n + 1)
    x = np.cos(np.arange(0, n + 1) * (np.pi / float(n)))
    d = np.ones(n + 1)
    d[0] = 2
    d[n] = 2

    # np.eye(n + 1) in next expression avoids div by zero.
    c = np.outer(d, (one / d)) / (np.outer(x, one) -
                                  np.outer(one, x) + np.eye(n + 1))

    #  Now fix diagonal and signs.
    c[0, 0] = (2 * n ** 2 + 1) / 6.0
    for i in range(1, n + 1):
        if ((i + 1) % 2) == 0:
            c[:, i] = -c[:, i]
            c[i, :] = -c[i, :]

        if i < n:
            c[i, i] = -x[i] / (2 * (1 - x[i] ** 2))
        else:
            c[n, n] = -c[0, 0]

    if k == 1:
        c = c[1::, 1::]

    return c



import numpy as np
from rogues.matrices import hankel


def ipjfact(n, k=0):
    """
    ipjfact   A Hankel matrix with factorial elements.
          a = ipjfact(n, k) is the matrix with
                    a(i,j) = (i+j)!    (k = 0, default)
                    a(i,j) = 1/(i+j)!  (k = 1)
          both are hankel matrices.
          The determinant and inverse are known explicitly.
          d = det(a) is returned is always returned as in
          a, d = ipjfact(n, k)

          Suggested by P. R. Graves-Morris.

          Reference:
          M.J.C. Gover, The explicit inverse of factorial Hankel matrices,
          Dept. of Mathematics, University of Bradford, 1993.
    """
    c = np.cumprod(np.arange(2, n + 2))
    d = np.cumprod(np.arange(n + 1, 2 * n + 1)) * c[n - 2]

    a = hankel(c, d)

    if k == 1:
        a = 1 / a

    d = 1

    #
    # There appears to be a bug in the implementaton of interger
    # multiply in numpy (note _not_ in Python).  Therefore we use
    # the explicit "cast" to float64 below.
    #
    if k == 0:
        for i in range(1, n):
            d = d * np.prod(np.arange(1, i + 2, dtype='float64')) *  \
                np.prod(np.arange(1, n - i + 1, dtype='float64'))

        d = d * np.prod(np.arange(1, n + 2, dtype='float64'))
    else:
        for i in range(0, n):
            d = d * np.prod(np.arange(1, i, dtype='float64')) /    \
                np.prod(np.arange(1, n + 1 + i, dtype='float64'))

        if (n * (n - 1) / 2) % 2:
            d = -d
    det_a = d

    return a, det_a



import numpy as np


class Higham(Exception):
    pass


def ge(b):
    """
    GE     Gaussian elimination without pivoting.
       [l, u, rho] = ge(a) computes the factorization a = l*u,
       where L is unit lower triangular and U is upper triangular.
       RHO is the growth factor.
       By itself, ge(a) returns the final reduced matrix from the
       elimination containing both L and U.

       Note added in porting to Python/numpy/scipy:
       --------------------------------------------
       There are obviously more efficient routines in numpy / scipy
       but this routine is intended for testing numerical properties,
       ie it can be used in the direct search method adsmax for finding
       a matrix that maximizes the value of rho. See the following very
       readable and fun paper that is available from Prof. Higham's web
       site:

        Reference:
        N.J. Higham, Optimization by direct search in matrix computations,
        SIAM J. Matrix Anal. Appl, 14(2): 317-333, April 1993.

    """
    a = b.copy()    # don't cream the input matrix
    n, n = a.shape
    maxA = a.max()
    rho = maxA

    for k in range(n - 1):

        if a[k, k] == 0:
            raise Higham('Elimination breaks down with zero pivot.')

        a[k + 1:n, k] = a[k + 1:n, k] / a[k, k]          # Multipliers.

        # Elimination
        a[k + 1:n, k + 1:n] = (a[k + 1:n, k + 1:n] -
                               np.outer(a[k + 1:n, k], a[k, k + 1:n]))
        rho = max(rho, (abs(a[k + 1:n, k + 1:n])).max())

    l = np.tril(a, -1) + np.eye(n)
    u = np.triu(a)
    rho = rho / maxA

    return l, u, rho



import numpy as np
from rogues.utils import tridiag


def lesp(n):
    """
    LESP   A tridiagonal matrix with real, sensitive eigenvalues.
       LESP(N) is an N-by-N matrix whose eigenvalues are real and smoothly
       distributed in the interval approximately [-2*N-3.5, -4.5].
       The sensitivities of the eigenvalues increase exponentially as
       the eigenvalues grow more negative.
       The matrix is similar to the symmetric tridiagonal matrix with
       the same diagonal entries and with off-diagonal entries 1,
       via a similarity transformation with d = np.diag(1!,2!,...,N!).

       References:
       H.W.J. Lenferink and M.N. Spijker, On the use of stability regions in
            the numerical analysis of initial value problems,
            Math. Comp., 57 (1991), pp. 221-237.
       L.N. Trefethen, Pseudospectra of matrices, in Numerical Analysis 1991,
            Proceedings of the 14th Dundee Conference,
            D.F. Griffiths and G.A. Watson, eds, Pitman Research Notes in
            Mathematics, volume 260, Longman Scientific and Technical, Essex,
            UK, 1992, pp. 234-266.
    """
    x = np.arange(2., n + 1)
    y = -(2. * np.arange(2, n + 2) + 1)
    t = tridiag(1 / x, y, x).todense()

    return t



import numpy as np
import numpy.linalg as nl
import numpy.random as nrnd


def qmult(b):
    """
    QMULT  Pre-multiply by random orthogonal matrix.
       QMULT(A) is Q*A where Q is a random real orthogonal matrix from
       the Haar distribution, of dimension the number of rows in A.
       Special case: if A is a scalar then QMULT(A) is the same as
                     QMULT(EYE(A)).

       Called by RANDSVD.

       Reference:
       G.W. Stewart, The efficient generation of random
       orthogonal matrices with an application to condition estimators,
       SIAM J. Numer. Anal., 17 (1980), 403-409.
    """
    try:
        n, m = b.shape
        a = b.copy()

    except AttributeError:
        n = b
        a = np.eye(n)

    d = np.zeros(n)

    for k in range(n - 2, -1, -1):
        # Generate random Householder transformation.
        x = nrnd.randn(n - k)
        s = nl.norm(x)
        # Modification to make sign(0) == 1
        sgn = np.sign(x[0]) + float(x[0] == 0)
        s = sgn * s
        d[k] = -sgn
        x[0] = x[0] + s
        beta = s * x[0]

        # Apply the transformation to a
        y = np.dot(x, a[k:n, :])
        a[k:n, :] = a[k:n, :] - np.outer(x, (y / beta))

    # Tidy up signs.
    for i in range(n - 1):
        a[i, :] = d[i] * a[i, :]

    # Now randomly change the sign (Gaussian dist)
    a[n - 1, :] = a[n - 1, :] * np.sign(nrnd.randn())

    return a



import numpy as np


def cpltaxes(zz, plt):
    """
    CPLTAXES   Determine suitable AXIS for plot of complex vector.
           X = CPLTAXES(Z), where Z is a complex vector,
           determines a 4-vector X such that AXIS(X) sets axes for a plot
           of Z that has axes of equal length and leaves a reasonable amount
           of space around the edge of the plot.

           Called by FV, GERSH, PS and PSCONT.

           NOTE: You must pass in the module instance of pylab in the
                 'plt' argument.
    """
    # Collapse down to a vector
    z = zz.ravel()

    # Set x and y axis ranges so both have the same length.

    xmin = min(z.real)
    xmax = max(z.real)
    ymin = min(z.imag)
    ymax = max(z.imag)

    # Fix for rare case of `trivial data'.
    if xmin == xmax:
        xmin = xmin - 1 / 2.
        xmax = xmax + 1 / 2.

    if ymin == ymax:
        ymin = ymin - 1 / 2.
        ymax = ymax + 1 / 2.

    if xmax - xmin >= ymax - ymin:
        ymid = (ymin + ymax) / 2.
        ymin = ymid - (xmax - xmin) / 2.
        ymax = ymid + (xmax - xmin) / 2.
    else:
        xmid = (xmin + xmax) / 2
        xmin = xmid - (ymax - ymin) / 2.
        xmax = xmid + (ymax - ymin) / 2.

    plt.axis('equal')

    # Scale ranges by 1+2*alpha to give extra space around edges of plot.

    alpha = 0.1
    x = np.zeros(4)
    x[0] = xmin - alpha * (xmax - xmin)
    x[1] = xmax + alpha * (xmax - xmin)
    x[2] = ymin - alpha * (ymax - ymin)
    x[3] = ymax + alpha * (ymax - ymin)

    if x[0] == x[1]:
        x[1] = x[1] + 0.1

    if x[2] == x[3]:
        x[3] = x[2] + 0.1

    return x



import numpy as np
import matplotlib.pylab as plt
from rogues.utils import cpltaxes


class Higham(Exception):
    pass


def gersh(A, plot=True):
    ''''
    gersh    Gershgorin disks.
             gersh(A) draws the Gershgorin disks for the square matrix A.
             The eigenvalues are plotted as crosses `x'.
             Alternative usage: [G, E] = gersh(a, False) suppresses the plot
             and returns the data in G, with A's eigenvalues in E.

             Try gersh(rg.lesp(n)) and gersh(rg.smoke(n))
    '''

    if not (A.ndim == 2 and np.diff(A.shape)[0] == 0):
        raise Higham("Matrix must be square.  It is: ", A.shape)

    n = len(A)
    m = 40
    G = np.zeros((m, n), dtype=np.complex128)

    d = np.diag(A)
    r = sum(np.abs(A - np.diag(d)).T).T
    e, _ = np.linalg.eig(A)

    radvec = np.exp(1j * np.linspace(0, 2 * np.pi, m))

    for j in range(n):
        G[:, j] = d[j] * np.ones(m) + r[j] * radvec

    if plot:
        ax = cpltaxes(G[:], plt)
        for j in range(n):
            plt.plot(G[:, j].real, G[:, j].imag, '-')      # Plot the disks.

        plt.plot(e.real, e.imag, 'x')    # Plot the eigenvalues too.
        plt.axis(ax)
        plt.axis('equal')
        plt.show()

    return G, e



import numpy as np
import scipy.linalg as sl


class Higham(Exception):
    pass


def condeig(a):
    """
    v, lambda, c = condeig(a) Computes condition numbers for the
    eigenvalues of a matrix. The condition numbers are the reciprocals
    of the cosines of the angles between the left and right eigenvectors.
    Inspired by Arno Onken's Octave code for condeig

    When checking against results obtained in Higham & Higham
    a = rogues.frank(6)
    lr, vr, c = matrixcomp.condeig(a)

    H & H get for lr  = [ 12.9736, 5.3832, 1.8355,  0.5448,  0.0771,  0.1858]
          and for  c  = [  1.3059, 1.3561, 2.0412, 15.3255, 43.5212, 56.6954]

    which they say that the small eigenvalues are slightly ill conditioned.
    With the this python/numpy condeig we get

    vr = [ 12.97360792, 5.38322318, 1.83552324, 0.54480378,   0.07707956,
            0.18576231]
     c = [  1.30589002, 1.35605093, 2.04115713, 15.32552609, 43.52124194,
           56.69535399]

    NOTE: we must use scipy.linalg.decomp.eig and _not_ np.linalg.eig
    """

    if len(a.shape) != 2:
        raise Higham("a must be a 2 dimensional array")
    else:
        m, n = a.shape
        if m != n or m < 2:
            raise Higham("a must be a square array with dimension of "
                         "2 or greater")

    # eigenvalues, left and right eigenvectors
    lamr, vl, vr = sl.eig(a, left=True, right=True)

    # Need to put the left eigenvectors into row form
    vl = vl.T

    # Normalize vectors
    for i in range(n):
        vl[i, :] = vl[i, :] / np.sqrt(abs(vl[i, :] ** 2).sum())

    # Condition numbers are reciprocal of the cosines (dot products) of the
    # left eignevectors with the right eigenvectors.   In a perfect world,
    # these numbers should all be one, but they are not.
    c = abs(1 / np.diag(np.dot(vl, vr)))

    return lamr, vr, c



import numpy as np


def vecperm(m, n=None):
    """
    VECPERM    Vec-permutation matrix.
           VECPERM(M, N) is the vec-permutation matrix, an MN-by-MN
           permutation matrix P with the property that if A is M-by-N then
           vec(A) = P*vec(A').
           If N is omitted, it defaults to M.

           P is formed by taking every n'th row from EYE(M*N), starting with
           the first and working down - see p. 277 of the reference.

           Reference:
           H. V. Henderson and S. R. Searle The vec-permutation matrix,
           the vec operator and Kronecker products: A review Linear and
           Multilinear Algebra, 9 (1981), pp. 271-288.
    """
    if n is None:
        n = m

    p = np.zeros((m * n, m * n))
    e = np.eye(m * n)

    k = 0
    for i in range(n):
        p[k:k + n, :] = e[i:m * n:n, :]
        k += n

    return p



def symmpart(a):
    """
    SYMMPART  Symmetric (Hermitian) part.
          SYMMPART(A) is the symmetric (Hermitian) part of A, (A + A')/2.
          It is the nearest symmetric (Hermitian) matrix to A in both the
          2- and the Frobenius norms.
    """
    s = (a + a.T) / 2.
    return s



import numpy as np
import numpy.linalg as nl
import numpy.random as nrnd
import numpy.testing as npt
import rogues


def test_augment():
    """One specific simple test for augment"""
    a = np.array([[1., 0., 0., 0., 4., 3., 2., 1.],
                  [0., 1., 0., 0., 3., 3., 2., 1.],
                  [0., 0., 1., 0., 0., 2., 2., 1.],
                  [0., 0., 0., 1., 0., 0., 1., 1.],
                  [4., 3., 0., 0., 0., 0., 0., 0.],
                  [3., 3., 2., 0., 0., 0., 0., 0.],
                  [2., 2., 2., 1., 0., 0., 0., 0.],
                  [1., 1., 1., 1., 0., 0., 0., 0.]])
    b = rogues.frank(4)
    c = rogues.augment(b)
    npt.assert_array_almost_equal(a, c, 12)


def test_bandred():
    """Simple Test of bandred, checks againts np.linalg.cond
    NEEDS MORE TESTING!  This test is pretty lame...
    """
    a = rogues.frank(9)
    # Use bandred to get a tri-diagonal form
    b = rogues.bandred(a, 1, 1)
    # Chop into a tri-diagonal form
    c = np.triu(np.tril(b, 1), -11)
    # They must be equal
    npt.assert_array_almost_equal(b, c)


def test_cgs():
    """Simple Test of cgs, classical Gram-Schmidt orthogonalization"""
    a = nrnd.rand(10, 10)
    q, r = rogues.cgs(a)
    # r must be upper triangular
    npt.assert_almost_equal(r, np.triu(r), 10)
    # factorization must hold
    npt.assert_almost_equal(a, q @ r, 10)


def test_cond():
    """Simple Test of cond, checks againts np.linalg.cond"""
    a = np.diag(np.arange(1, 11)) + np.diag(np.ones(9), 1)
    c1 = rogues.cond(a)
    c2 = np.linalg.cond(a)
    npt.assert_almost_equal(c1, c2, 10)


def test_condeig():
    """
    Simple Test of condeig, checks (partly) results in Higham & Higham
    Note that the last two condtion numbers do _not_ match the results
    of H&H and seem exceptionally large
    """
    a = rogues.frank(6)
    gold_c = [1.30589002, 1.35605093, 2.04115713, 15.32552609, \
              43.52124194, 56.69535399]
    lr, vr, c = rogues.condeig(a)

    for i in range(6):
        npt.assert_almost_equal(c[i], gold_c[i], 6)

# @npt.dec.knownfailureif(True, "diagpiv hasn't been debugged yet!")
# def test_diagpiv():
#     """Simple exercise of diagpiv"""
#     l,d,p,rho = rogues.diagpiv(5)
#     l,u,r = ge(a)
#     b = l @ u
#     npt.assert_array_almost_equal(a, b, 12)


def test_dual():
    """Simple Test of dual, checks the the product of the norms"""
    p = 3
    q = 1 / (1. - 1. / float(p))
    x = nrnd.rand(20)
    y = rogues.dual(x, p)
    npt.assert_almost_equal(np.dot(x, y), nl.norm(x, p) * nl.norm(y, q), 10)


def test_ge():
    """Simple exercise of the ge (Gaussian elimination)"""
    a = rogues.gfpp(5)
    l, u, r = rogues.ge(a)
    b = l @ u
    npt.assert_array_almost_equal(a, b, 12)

# tested in rogues/matrices/tests
# def test_hankel():
#     """
#     Simple test of hankel matrix.
#     """
#     a = np.arange(10)
#     b = np.arange(10, 20)
#     h = rogues.hankel(a, b)
#     g = np.array([[0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
#                   [1, 2, 3, 4, 5, 6, 7, 8, 9, 11],
#                   [2, 3, 4, 5, 6, 7, 8, 9, 11, 12],
#                   [3, 4, 5, 6, 7, 8, 9, 11, 12, 13],
#                   [4, 5, 6, 7, 8, 9, 11, 12, 13, 14],
#                   [5, 6, 7, 8, 9, 11, 12, 13, 14, 15],
#                   [6, 7, 8, 9, 11, 12, 13, 14, 15, 16],
#                   [7, 8, 9, 11, 12, 13, 14, 15, 16, 17],
#                   [8, 9, 11, 12, 13, 14, 15, 16, 17, 18],
#                   [9, 11, 12, 13, 14, 15, 16, 17, 18, 19]])

#     npt.assert_array_equal(h, g)


def test_house():
    """
    Simple test of house (Householder transform). Just checks to
    see if the det(h) == -1, (reflection matrices have this property)
    """
    x = np.ones(5)
    v, beta, s = rogues.house(x)
    h = np.eye(5) - beta * np.outer(v, v)
    d = nl.det(h)
    npt.assert_almost_equal(d, -1., 12)


def test_house_reflect():
    """
    Simple test of house (Householder transform). Just checks to
    see if the input vector is reflected to be all in the x[0] direction
    """
    x = np.ones(5)
    v, beta, s = rogues.house(x)
    h = np.eye(5) - beta * np.outer(v, v)
    d = np.dot(h, x)
    d2 = np.dot(d, d)
    npt.assert_almost_equal(d2, d[0] ** 2, 12)


def test_mgs():
    """Simple test modified gram schmidt orthogonalization"""
    a = np.random.randn(10, 10)
    q, r = rogues.mgs(a)
    b = q @ q.T
    npt.assert_array_almost_equal(b, np.eye(10), 12)


def test_repmat():
    """
    Simple test of repmat. (repeated matrix)
    """
    a = np.array([2, 3, 4])
    b = rogues.repmat(a, 3, 4)
    c = np.array([[2, 3, 4, 2, 3, 4, 2, 3, 4, 2, 3, 4],
                  [2, 3, 4, 2, 3, 4, 2, 3, 4, 2, 3, 4],
                  [2, 3, 4, 2, 3, 4, 2, 3, 4, 2, 3, 4]])
    npt.assert_array_equal(b, c)


def test_rq():
    """Simple test of the Raleigh quotient (x.(A.x))/(x.x)"""
    n = 10
    a = np.eye(n)
    b = np.ones(n)
    npt.assert_equal(rogues.rq(a, b), 1)


def test_skewpart():
    """Simple test of taking the skew part of a matrix"""
    a = nrnd.rand(10, 10)
    s = rogues.skewpart(a)
    npt.assert_array_almost_equal(s, -s.T)


def test_sparsify():
    """Simple test of sparsify'ing a matrix."""
    n = 1000
    prob = 0.25
    a = np.ones((n, n))
    s = rogues.sparsify(a, prob)
    assert(s.sum() > n * n * (1 - 1.05 * prob))


def test_sub():
    """Simple test of extracting the principal submatrix."""
    n = 10
    a = np.zeros((n, n))
    b = 5 * np.ones((3, 3))
    a[0:3, 0:3] = b
    s = rogues.sub(a, 3)
    npt.assert_array_equal(b, s)
    b = np.zeros((3, 3))
    b[0, 0] = 5
    s = rogues.sub(a, 2, 5)
    npt.assert_array_equal(b, s)


def test_symmpart():
    """Simple test of extracting the symmetric part of a matrix."""
    n = 10
    a = nrnd.rand(n, n)
    b = rogues.symmpart(a)
    e, w = nl.eig(b)
    z = np.zeros(n)
    npt.assert_array_equal(e.imag, z)


def test_toeplitz():
    """
    Simple test of toeplitz. This just checks that the
    known values of  determinant for a simple form at a
    succession of matrix sizes
    """
    dets = []
    for i in range(3, 30):
        a = np.arange(1, i)
        b = rogues.toeplitz(a)
        dets.append(nl.det(b))
    dets = np.array(dets)

    # Here are the known values of the determinants
    ans  = np.array([(-1) ** (i - 1) * (i + 1) * (2 ** (i - 2)) for i in \
                       range(2, 29)])

    # pretty bad round off for some values
    npt.assert_array_almost_equal(dets, ans, 5)


def test_treshape_0():
    """Simple test of treshape_0"""
    n = 4
    m = n * (n + 1) / 2
    a = np.arange(1, m + 1)
    b = rogues.treshape(a)
    c = rogues.treshape(b, unit=2)
    npt.assert_array_equal(a, c)


def test_treshape_1():
    """Simple test of treshape_1"""
    n = 4
    m = n * (n + 1) / 2
    a = np.arange(1, m + 1)

    d = rogues.treshape(a, unit=1)
    e = rogues.treshape(d, unit=3)
    npt.assert_array_equal(a, e)
    npt.assert_equal(nl.det(d), 1.0)


def test_treshape_2():
    """Simple test of treshape_2"""
    n = 4
    m = n * (n + 1) / 2
    a = np.arange(1, m + 1)
    b = rogues.treshape(a, row_wise=True)
    c = rogues.treshape(b, unit=2, row_wise=True)
    npt.assert_array_equal(a, c)


def test_treshape_3():
    """Simple test of treshape_3"""
    n = 4
    m = n * (n + 1) / 2
    a = np.arange(1, m + 1)

    d = rogues.treshape(a, unit=1, row_wise=True)
    e = rogues.treshape(d, unit=3, row_wise=True)
    npt.assert_array_equal(a, e)
    npt.assert_equal(nl.det(d), 1.0)


def test_tridiag_a():
    """Simple test of tridiag.  Just recover the diagonals"""
    n = 10           # size of array
    x = []
    x.append(np.ones(n - 1))
    x.append(2 * np.ones(n))
    x.append(3 * np.ones(n - 1))
    a = rogues.tridiag(x[0], x[1], x[2])
    b = a.todense()

    npt.assert_array_equal(x[0], np.diag(b, -1))
    npt.assert_array_equal(x[1], np.diag(b, 0))
    npt.assert_array_equal(x[2], np.diag(b, 1))


def test_tridiag_b():
    """Simple test of tridiag. Check the const diags from single arg"""
    n = 10
    a = rogues.tridiag(n)
    b = a.todense()
    x = []
    x.append(-1 * np.ones(n - 1))
    x.append(2 * np.ones(n))
    x.append(-1 * np.ones(n - 1))
    npt.assert_array_equal(x[0], np.diag(b, -1))
    npt.assert_array_equal(x[1], np.diag(b, 0))
    npt.assert_array_equal(x[2], np.diag(b, 1))


def test_tridiag_c():
    """Simple test of tridiag. Check with 4 scalar args and known
    eigenvalues"""
    c = 1
    d = 2
    e = 3
    n = 10
    a = rogues.tridiag(n, c, d, e)
    b = a.todense()
    w, v = nl.eig(b)
    w.sort()
    tw = np.array([np.cos(i * np.pi / (n + 1)) for i in range(1, n + 1)])
    tw = d + 2 * np.sqrt(c * e) * tw
    tw.sort()
    res = nl.norm(w - tw)
    npt.assert_almost_equal(res, 0.0, 12)


def test_vand():
    """Simple test of vand."""
    n = 10
    a = np.arange(1, n)
    b = rogues.vand(a)
    c = np.vander(a)
    npt.assert_equal(b, c.T[::-1, :])


def test_vecperm():
    """Simple test of vecperm."""
    n = 5
    a = rogues.vecperm(n)
    npt.assert_equal(nl.det(a), 1)
    npt.assert_equal(a.sum(), n * n)

if __name__ == "__main__":
    npt.run_module_suite()


#IgnoreException
#NumpyTest
#NumpyTestCase
#ParametricTestCase
#TestCase
#Tester
#__builtins__
#__doc__
#__file__
#__name__
#__path__
#assert_almost_equal
#assert_approx_equal
#assert_array_almost_equal
#assert_array_equal
#assert_array_less
#assert_equal
#assert_raises
#assert_string_equal
#build_err_msg
#dec
#decorate_methods
#decorators
#importall
#jiffies
#measure
#memusage
#nosetester
#numpytest
#parametric
#print_assert_equal
#raises
#rand
#restore_path
#run_module_suite
#rundocs
#runstring
#set_local_path
#set_package_path
#test
#utils
#verbose



import numpy as np


def grcar(n, k=3):
    """
    GRCAR     Grcar matrix - a Toeplitz matrix with sensitive eigenvalues.
          GRCAR(N, K) is an N-by-N matrix with -1s on the
          subdiagonal, 1s on the diagonal, and K superdiagonals of 1s.
          The default is K = 3.  The eigenvalues of this matrix form an
          interesting pattern in the complex plane (try ps(grcar(32))).
           or    a   = grcar(100)
                 w,v = np.eig(a)
                 pl.plot(w.real(), w.imag())

          References:
          J.F. Grcar, Operator coefficient methods for linear equations,
               Report SAND89-8691, Sandia National Laboratories, Albuquerque,
               New Mexico, 1989 (Appendix 2).
          N.M. Nachtigal, L. Reichel and L.N. Trefethen, A hybrid GMRES
               algorithm for nonsymmetric linear systems, SIAM J. Matrix Anal.
               Appl., 13 (1992), pp. 796-825.
    """

    g = np.tril(np.triu(np.ones((n, n))), k) - np.diag(np.ones(n - 1), -1)

    return g



import numpy as np


def cgs(a):
    """
    CGS     Classical Gram-Schmidt QR factorization.
    [Q, R] = cgs(A) uses the classical Gram-Schmidt method to compute the
    factorization A = Q*R for m-by-n A of full rank,
    where Q is m-by-n with orthonormal columns and R is n-by-n.

    NOTE: This was ported just for an example of the Gram-Schmidt
    orthogonalization. If you really want the QR factorization, you
    should probably use numpy.linalg.qr
    """
    m, n = a.shape
    q = np.zeros((m, n))
    r = np.zeros((n, n))

    # Gram-Schmidt expressed in matrix-vector form.
    # Treat j = 0 as special case to avoid if tests inside loop.
    r[0, 0] = np.linalg.norm(a[:, 0])
    q[:, 0] = a[:, 0] / r[0, 0]
    for j in range(1, n):
        r[0:j - 1, j] = np.dot(q[:, 0:j - 1].T, a[:, j])
        temp = a[:, j] - np.dot(q[:, 0:j - 1], r[0:j - 1, j])
        r[j, j] = np.linalg.norm(temp)
        q[:, j] = temp / r[j, j]

    return q, r



import numpy as np


def clement(n, k=0):
    """
    CLEMENT   Clement matrix - tridiagonal with zero diagonal entries.
          CLEMENT(N, K) is a tridiagonal matrix with zero diagonal entries
          and known eigenvalues.  It is singular if N is odd.  About 64
          percent of the entries of the inverse are zero.  The eigenvalues
          are plus and minus the numbers N-1, N-3, N-5, ..., (1 or 0).
          For K = 0 (the default) the matrix is unsymmetric, while for
          K = 1 it is symmetric.
          CLEMENT(N, 1) is diagonally similar to CLEMENT(N).

          Similar properties hold for TRIDIAG(X,Y,Z) where Y = ZEROS(N,1).
          The eigenvalues still come in plus/minus pairs but they are not
          known explicitly.

          References:
          P.A. Clement, A class of triple-diagonal matrices for test
             purposes, SIAM Review, 1 (1959), pp. 50-52.
          A. Edelman and E. Kostlan, The road from Kac's matrix to Kac's
             random polynomials. In John~G. Lewis, editor, Proceedings of
             the Fifth SIAM Conference on Applied Linear Algebra Society
             for Industrial and Applied Mathematics, Philadelphia, 1994,
             pp. 503-507.
          O. Taussky and J. Todd, Another look at a matrix of Mark Kac,
             Linear Algebra and Appl., 150 (1991), pp. 341-360.
    """
    z = np.arange(1, n)

    if k == 0:
        a = np.diag(z[::-1], -1) + np.diag(z, 1)
    else:
        y = np.sqrt(z[::-1] * z)
        a = np.diag(y, -1) + np.diag(y, 1)

    return a



from rogues.matrices import hilb


def invol(n):
    """
    invol(n)   an involutory matrix of order n.
        a = invol(n) is an n-by-n involutory (a*a = eye(n)) and
        ill-conditioned matrix.
        It is a diagonally scaled version of hilb(n).
        nb: b = (eye(n)-a)/2 and b = (eye(n)+a)/2 are idempotent (b*b = b).

        Reference:
        A.S. Householder and J.A. Carpenter, The singular values
        of involutory and of idempotent matrices, Numer. Math. 5 (1963),
        pp. 234-237.
    """

    a = hilb(n)

    d = -n
    a[:, 0] = d * a[:, 0]

    for i in range(n - 1):
        d = -(n + i + 1) * (n - i - 1) * d / float((i + 1) * (i + 1))
        a[i + 1, :] = d * a[i + 1, :]

    return a



import numpy as np


def kms(n, rho=0.5):
    """
    kms   Kac-Murdock-Szego Toeplitz matrix.
      a = kms(n, rho) is the n-by-n Kac-Murdock-Szego Toeplitz matrix with
      a(i,j) = rho**(abs((i-j))) (for real rho).
      If RHO is complex, then the same formula holds except that elements
      below the diagonal are conjugated.
      RHO defaults to 0.5.
      Properties:

         a has an LDL' factorization with
                  L = INV(TRIW(N,-RHO,1)'),
                  D(i,i) = (1-ABS(RHO)^2)*EYE(N) except D(1,1) = 1.
            NOTE: I have not been able to verify this property. See
            test_rogues.py for more details.

         a is positive definite if and only if 0 < ABS(RHO) < 1.

         INV(A) is tridiagonal.
            NOTE: verified on several examples and used in unit test.

       Reference:
       W.F. Trench, Numerical solution of the eigenvalue problem
       for Hermitian Toeplitz matrices, SIAM J. Matrix Analysis and Appl.,
       10 (1989), pp. 135-146 (and see the references therein).
    """
    a = np.outer(np.arange(1, n + 1), np.ones(n))
    a = np.abs(a - a.T)
    a = rho ** a
    if np.iscomplex(rho):
        a = np.conj(np.tril(a, -1)) + np.triu(a)

    return a



import numpy as np


class Higham(Exception):
    pass


def repmat(a, nrows, ncols):
    """
    repmat(a, nrows, ncols)
    Simple implementation of m*lab's repmat function.
    """
    c = np.kron(np.ones((nrows, ncols)), a)
    return c



import numpy as np


def smoke(n, k=0):
    """
    smoke     smoke matrix - complex, with a `smoke ring' pseudospectrum.
          smoke(n) is an n-by-n matrix with 1s on the
          superdiagonal, 1 in the (n,1) position, and powers of
          roots of unity along the diagonal.
          smoke(n, 1) is the same except for a zero (n,1) element.
          the eigenvalues of smoke(n, 1) are the n'th roots of unity;
          those of smoke(n) are the n'th roots of unity times 2^(1/n).

          Try ps(smoke(32)).  For smoke(n, 1) the pseudospectrum looks
          like a sausage folded back on itself.
          gersh(smoke(n, 1)) is interesting.

          Reference:
          L. Reichel and L.N. Trefethen, Eigenvalues and pseudo-eigenvalues of
          Toeplitz matrices, Linear Algebra and Appl., 162-164:153-185, 1992.
    """
    w = np.exp(2 * np.pi * 1j / n)
    a = (np.diag(np.hstack((w ** np.arange(1, n), 1))) +
         np.diag(np.ones(n - 1), 1))
    if k == 0:
        a[n - 1, 0] = 1

    return a



import numpy as np


def gearm(n, i=None, j=None):
    """
    GEARM   Gear matrix.
        A = GEARM(N,I,J) is the N-by-N matrix with ones on the sub- and
        super-diagonals, SIGN(I) in the (1,ABS(I)) position, SIGN(J)
        in the (N,N+1-ABS(J)) position, and zeros everywhere else.
        Defaults: i = n - 1, j = -(n - 1).  Note we use the zero based
        indexing for i and j.
        All eigenvalues are of the form 2*COS(a) and the eigenvectors
        are of the form [SIN(w+a), SIN(w+2a), ..., SIN(w+Na)].
        The values of a and w are given in the reference below.
        A can have double and triple eigenvalues and can be defective.
        GEARM(N) is singular.

        (GEAR is a Simulink function, hence GEARM for Gear matrix.)
        Reference:
        C.W. Gear, A simple set of test matrices for eigenvalue programs,
        Math. Comp., 23 (1969), pp. 119-125.
    """
    if i is None:
        i = n - 1
        j = -(n - 1)

    if not(abs(i) < n and abs(j) < n):
        raise ValueError('Invalid i and j parameters')

    a = np.diag(np.ones(n - 1), -1) + np.diag(np.ones(n - 1), 1)
    a[0, np.abs(i)] = np.sign(i)
    a[n - 1, n - 1 - np.abs(j)] = np.sign(j)

    return a



import numpy as np
from rogues.utils.house import house


class Higham(Exception):
    pass


def bandred(x, kl=0, ku=-1):
    """
    BANDRED  Band reduction by two-sided unitary transformations.
         B = BANDRED(A, KL, KU) is a matrix unitarily equivalent to A
         with lower bandwidth KL and upper bandwidth KU
         (i.e. B(i,j) = 0 if i > j + KL or j > i + KU).
         The reduction is performed using Householder transformations.
         If KU is omitted it defaults to KL.

         Called by RANDSVD.
         This is a `standard' reduction.  Cf. reduction to bidiagonal form
         prior to computing the SVD.  This code is a little wasteful in that
         it computes certain elements which are immediately set to zero!

         Reference:
         G.H. Golub and C.F. Van Loan, Matrix Computations, second edition,
         Johns Hopkins University Press, Baltimore, Maryland, 1989.
         Section 5.4.3.
    """
    # Work on a local copy of the array.
    a = x.copy()

    if kl == 0 and (ku == 0 or ku == -1):
        raise Higham("You've asked for a diagonal matrix. "
                     "In that case use the SVD!")
    elif ku == -1:
        # set ku to k1 if it has not yet been set
        ku = kl

    # Check for special case where order of left/right transformations matters.
    # Easiest approach is to work on the transpose, flipping back at the end.
    flip = 0
    if ku == 0:
        a = a.T
        kl, ku = ku, kl
        flip = 1

    m, n = a.shape

    for j in range(min(min(m, n), max(m - kl - 1, n - ku - 1))):
        if j + kl + 1 <= m:
            v, beta, s = house(a[j + kl:m, j])
            temp = a[j + kl:m, j:n]
            a[j + kl:m, j:n] = temp - beta * np.outer(v, np.dot(v, temp))
            a[j + kl + 1:m, j] = np.zeros(m - j - kl - 1)

        if j + ku + 1 <= n:
            v, beta, s = house(a[j, j + ku:n])
            temp = a[j:m, j + ku:n]
            a[j:m, j + ku:n] = temp - beta * np.outer(np.dot(temp, v), v)
            a[j, j + ku + 1:n] = np.zeros(n - j - ku - 1)

    if flip == 1:
        a = a.T

    return a



import numpy as np
import warnings


def hankel(a, b=None):
    """
    hankel(a) returns a toeplitz matrix given "a", the first row of the
    matrix.  This matrix is defined as:

         [[   a[0], a[1], a[2], a[3], ..., a[n-1], a[n] ]]
          [   a[1], a[2], a[3], a[4], ...,   a[n],   0  ]
          [   a[2], a[3], a[4], ...,     ]
          ...
          [ a[n-2], a[n-1], a[n],  0,   ...
          [ a[n-1], a[n],      0,  0,   ...           0
          [   a[n],    0,      0,  0,   ...           0 ] ]


    Note that all the non-zero anti-diagonals are constant

    If called as hankel(a, b) then create the hankel matrix where a is the
    first column and b is the last row.  If a[-1] != b[0], a[-1] is chosen
    but a warning message is printed.  For example

    >>> import numpy as np
    >>> from rogues import hankel
    >>> a = np.arange(10)
    >>> b = np.arange(10,20)
    >>> h = hankel(a, b)
    Warning: a[-1] != b[0]. a[-1] is chosen
    >>> h
    array([[ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9],
           [ 1,  2,  3,  4,  5,  6,  7,  8,  9, 11],
           [ 2,  3,  4,  5,  6,  7,  8,  9, 11, 12],
           [ 3,  4,  5,  6,  7,  8,  9, 11, 12, 13],
           [ 4,  5,  6,  7,  8,  9, 11, 12, 13, 14],
           [ 5,  6,  7,  8,  9, 11, 12, 13, 14, 15],
           [ 6,  7,  8,  9, 11, 12, 13, 14, 15, 16],
           [ 7,  8,  9, 11, 12, 13, 14, 15, 16, 17],
           [ 8,  9, 11, 12, 13, 14, 15, 16, 17, 18],
           [ 9, 11, 12, 13, 14, 15, 16, 17, 18, 19]])
    """
    # Error checking...
    try:
        m, = a.shape
        if b is not None:
            n, = b.shape
    except (ValueError, AttributeError):
        raise ValueError("Input arrays must be one dimensional")

    if b is None:
        b = np.zeros_like(a)
        n = m
    elif a[-1] != b[0]:
        warnings.warn("a[-1] != b[0]. a[-1] is chosen")

    k = np.hstack((a, b[1:]))
    h = k[:n]
    for i in range(1, m):
        h = np.vstack((h, k[i:i + n]))

    return h



import numpy as np


def minij(n):
    """
    MINIJ   Symmetric positive definite matrix MIN(i,j).
        A = MINIJ(N) is the N-by-N symmetric positive definite matrix with
        A(i,j) = MIN(i,j).
        Properties, variations:
        INV(A) is tridiagonal: it is minus the second difference matrix
                    except its (N,N) element is 1.
        2*A-ONES(N) (Givens' matrix) has tridiagonal inverse and
                    eigenvalues .5*sec^2([2r-1)PI/4N], r=1:N.
        (N+1)*ONES(N)-A also has a tridiagonal inverse.

        References:
        J. Todd, Basic Numerical Mathematics, Vol. 2: Numerical Algebra,
           Birkhauser, Basel, and Academic Press, New York, 1977, p. 158.
        D.E. Rutherford, Some continuant determinants arising in physics and
           chemistry---II, Proc. Royal Soc. Edin., 63, A (1952), pp. 232-241.
           (For the eigenvalues of Givens' matrix.)
    """
    o = np.outer(np.ones(n), np.arange(1, n + 1))
    ot = o.T
    a = np.where(o < ot, o, ot)
    return a



import numpy as np


class Higham(Exception):
    pass


def rq(A, x):
    """
    rg      Rayleigh quotient.
            rq(a, x) is the Rayleigh quotient of a and x, x'*A*x/(x'*x).

        Called by FV.
        NOTE: This function has a name clash with scipy.linalg.rq which
        computes the RQ decomposition of a matrix.
    """
    z = np.dot(x.T, np.dot(A, x)) / np.dot(x.T, x)

    return z.item()



import numpy as np
import numpy.linalg as nl

status_file = None
trace = False


def print_status(msg):
    if trace and status_file is not None:
        status_file.write(msg)


def mdsmax(fun, x, stopit=None, savit=None, varargin=[]):
    """
    MDSMAX  Multidirectional search method for direct search optimization.
        [x, fmax, nf] = MDSMAX(FUN, x0, STOPIT, SAVIT) attempts to
        maximize the function FUN, using the starting vector x0.
        The method of multidirectional search is used.
        Output arguments:
               x    = vector yielding largest function value found,
               fmax = function value at x,
               nf   = number of function evaluations.
        The iteration is terminated when either
               - the relative size of the simplex is <= stopit[0]
                 (default 1e-3),
               - stopit[1] function evaluations have been performed
                 (default inf, i.e., no limit), or
               - a function value equals or exceeds stopit[2]
                 (default inf, i.e., no test on function values).
        The form of the initial simplex is determined by STOPIT[3]:
          stopit[3] = 0: regular simplex (sides of equal length, the default),
          stopit[3] = 1: right-angled simplex.
        Progress of the iteration is not shown if stopit[4] = 0 (default 1).
        If a non-empty fourth parameter string SAVIT is present, then
        `SAVE SAVIT x fmax nf' is executed after each inner iteration.
        NB: x0 can be a matrix.  In the output argument, in SAVIT saves,
            and in function calls, x has the same shape as x0.
        MDSMAX(fun, x0, STOPIT, SAVIT, P1, P2,...) allows additional
        arguments to be passed to fun, via feval(fun,x,P1,P2,...).

    This implementation uses 2n^2 elements of storage (two simplices), where x0
    is an n-vector.  It is based on the algorithm statement in [2, sec.3],
    modified so as to halve the storage (with a slight loss in readability).

    References:
    [1] V. J. Torczon, Multi-directional search: A direct search algorithm for
        parallel machines, Ph.D. Thesis, Rice University, Houston, Texas, 1989.
    [2] V. J. Torczon, On the convergence of the multidirectional search
        algorithm, SIAM J. Optimization, 1 (1991), pp. 123-145.
    [3] N. J. Higham, Optimization by direct search in matrix computations,
        SIAM J. Matrix Anal. Appl, 14(2): 317-333, 1993.
    [4] N. J. Higham, Accuracy and Stability of Numerical Algorithms,
        Second edition, Society for Industrial and Applied Mathematics,
        Philadelphia, PA, 2002; sec. 20.5.
    [5] T.G. Kolda, R. M. Lewis, V. Torczon, Optimization by Direct Search: New
        Perspectives on Some Classical and Modern Methods", SIAM Review,
        Vol. 45, No. 3, pp. 385-482
    """
    global status_file
    global trace

    # Stopping with 1e-3 actually leads to wrong results..
    #
    if stopit is None:
        stopit = (1e-5, np.inf, np.inf, 0, 1)

    x0 = x
    n = x0.size

    varargin = []

    mu = 2             # Expansion factor.
    theta = 0.5        # Contraction factor.

    # Set up convergence parameters etc.
    # Tolerance for cgce test based on relative size of simplex.
    tol = stopit[0]
    trace = stopit[4]

    if savit is not None:
        status_file = open(savit, 'w')

    v = np.hstack((np.zeros((n, 1)), np.eye(n)))
    t = v
    f = np.zeros(n + 1)
    ft = f
    v[:, 0] = x0
    f[0] = fun(x, varargin)
    fmax_old = f[0]

    print_status('f(x0) = %9.4e\n' % f[0])

    # Set up initial simplex.
    scale = max(nl.norm(x0, np.inf), 1)
    if stopit[3] == 0:
        # Regular simplex - all edges have same length
        # Generated from construction given in reference [18, pp. 80-81] of [1]
        alpha = scale / (n * np.sqrt(2)) * np.hstack((np.sqrt(n + 1) - 1 + n,
                                                     np.sqrt(n + 1) - 1))
        v[:, 1:n + 1] = np.outer(x0 + alpha[1] * np.ones(n), np.ones(n))
        for j in range(1, n + 1):
            v[j - 1, j] = x0[j - 1] + alpha[0]
            x = v[:, j]
            f[j] = fun(x, varargin)

    else:
        # Right-angled simplex based on co-ordinate axes.
        alpha = scale * np.ones(n + 1)
        for j in range(1, n + 1):
            v[:, j] = x0 + alpha[j] * v[:, j]
            x = v[:, j]
            f[j] = fun(x, varargin)

    nf = n + 1
    size = 0           # Integer that keeps track of expansions/contractions.
    flag_break = 0     # Flag which becomes true when ready to quit outer loop.

    k = 0
    m = 0

    while True:
        k = k + 1

        # Find a new best vertex  x  and function value  fmax = f(x).
        fmax = np.max(f)
        j = np.argmax(f)
        # swap_columns 0, j
        v[:, [0, j]] = v[:, [j, 0]]
        v0 = v[:, 0]
        if savit is not None:
            x = v0
            print_status(' x: %2.6e  fmax: %2.6e  nf: %d\n' % (x, fmax, nf))

        f[0], f[j] = f[j], f[0]

        print_status('Iter. %2.0f,  inner = %2.0f,  size = %2.0f,  ' %
                     (k, m, size))

        print_status('nf = %3.0f,  f = %9.4e  (%2.1f)\n' %
                     (nf, fmax, 100 * (fmax - fmax_old) /
                      (abs(fmax_old) + np.finfo(float).eps)))

        fmax_old = fmax

        # Stopping Test 1 - f reached target value?
        if fmax >= stopit[2]:
            msg = 'Exceeded target...quitting\n'
            break

        m = 0

        while True:
            m = m + 1

            # Stopping Test 2 - too many f-evals?
            if nf >= stopit[1]:
                msg = 'Max number of function evaluations exceeded...quitting'
                flag_break = 1
                break

            # Stopping Test 3 - converged?   This is test (4.3) in [1].
            vt = np.outer(v0, np.ones(n))
            # size_simplex = nl.norm(v[:,1:n+1] - v0[:,np.ones(n)], 1) /  \
            size_simplex = (nl.norm(v[:, 1:n + 1] - vt, 1) /
                            max(1, nl.norm(v0, 1)))
            if size_simplex <= tol:
                msg = 'Simplex size %9.4e <= %9.4e...quitting\n' % \
                               (size_simplex, tol)
                flag_break = 1
                break

            for j in range(1, n + 1):      # ---Rotation (reflection) step.
                t[:, j] = 2 * v0 - v[:, j]
                x = t[:, j]
                ft[j] = fun(x, varargin)

            nf = nf + n

            replaced = (np.max(ft[1:n + 1]) > fmax)

            if replaced:
                for j in range(1, n + 1):   # ---Expansion step.
                    v[:, j] = (1 - mu) * v0 + mu * t[:, j]
                    x = v[:, j]
                    f[j] = fun(x, varargin)

                nf = nf + n
                # Accept expansion or rotation?
                if np.max(ft[1:n + 1]) > np.max(f[1:n + 1]):
                    # Accept rotation.
                    v[:, 1:n + 1] = t[:, 1:n + 1]
                    f[1:n + 1] = ft[1:n + 1]
                else:
                    size += 1    # Accept expansion (f and V already set).

            else:
                for j in range(1, n + 1):      # ---Contraction step.
                    v[:, j] = (1 + theta) * v0 - theta * t[:, j]
                    x = v[:, j]
                    f[j] = fun(x, varargin)

                nf = nf + n
                replaced = (np.max(f[1:n + 1]) > fmax)
                # Accept contraction (f and V already set).
                size -= 1

            if replaced:
                break

            if m % 10 == 0:
                print_status('        ...inner = %2.0f...\n' % m)

        if flag_break:
            break

    # Finished.
    print_status(msg)
    x = v0

    return x, fmax, nf



import numpy as np


class Higham(Exception):
    pass


def house(x):
    """
    house(x)   Householder matrix.
        If
           v, beta = house(x) then
           h  = eye - beta* np.outer(v, v)  is a Householder matrix such that
           Hx = -sign(x[0]) * norm(x) * e_1

        NB: If x = 0 then v = 0, beta = 1 is returned.
            x can be real or complex.
            sign(x) := exp(i*arg(x)) ( = x / abs(x) when x ~= 0).

        Theory: (textbook references Golub & Van Loan 1989, 38-43;
                 Stewart 1973, 231-234, 262; Wilkinson 1965, 48-50;
                 Higham 2002, 354-355)
        Hx = y: (I - beta*v*v')x = -s*e_1.
        Must have |s| = norm(x), v = x+s*e_1, and
        x'y = x'Hx =(x'Hx)' real => arg(s) = arg(x(1)).
        So take s = sign(x(1))*norm(x) (which avoids cancellation).
        v'v = (x(1)+s)^2 + x(2)^2 + ... + x(n)^2
            = 2*norm(x)*(norm(x) + |x(1)|).

        References:
        G.H. Golub and C.F. Van Loan, Matrix Computations, Third edition,
           Johns Hopkins University Press, Baltimore, Maryland, 1996.
        G.W. Stewart, Introduction to Matrix Computations, Academic Press,
           New York, 1973,
        J.H. Wilkinson, The Algebraic Eigenvalue Problem, Oxford University
           Press, 1965.
        N.J. Higham, Accuracy and Stability of Numerical Algorithms, SIAM,
           Philadelphia, 2002
    """
    if len(x.shape) == 1:
        n, = x.shape
    elif len(x.shape) == 2:
        n, m = x.shape
        if m != 1:
            raise Higham("x must be a vector")
    else:
        raise Higham("x must be a vector")

    # In numpy sign(0) = 0.0 and it looks like that might be the same in
    # m*lab.  Here we need sign(0) to be 1, rather than zero, which is the
    # reason for the code below.
    #
    sg = 1.0
    if np.sign(x[0]) < 0:
        sg = np.sign(x[0])

    s = sg * np.linalg.norm(x)
    v = x.copy()            # must copy or else x[0] get creamed

    # Quit if x is the zero vector.
    if s == 0:
        beta = 1
        return

    v[0] += s

    try:
        s = s.conjugate()
    except AttributeError:
        pass

    beta = 1 / (s * v[0])                           # NB the conjugated s.

    # beta = 1/(abs(s)*(abs(s)+abs(x(1)) would guarantee beta real.
    # But beta as above can be non-real (due to rounding) only when
    # x is complex.

    return v, beta, s



import numpy as np
from rogues.utils import tridiag


def neumann(n):
    """
    neumann  Singular matrix from the discrete Neumann problem (sparse).
         neumann(n) is the singular, row diagonally dominant matrix resulting
         from discretizing the neumann problem with the usual five point
         operator on a regular mesh.
         It has a one-dimensional null space with null vector ones(n,1).
         the dimension n should be a perfect square, or else a 2-vector,
         in which case the dimension of the matrix is n[1]*n[2].

         a, t = neumann(n) where a is the singular matrix of a*u = b
         which is the finite difference analog of the Neumann problem:

                                 del(u) = -f   in  R
                                 du/dn  =  g   on dR

         The matrix t corresponds to a specific ordering on a regular mesh
         of the finite differences where

                                a = t @ I + I @ t

         where I is the identity matrix and @ denotes the Kronecker product.
         The matrix t is tri-diagonal whose rows sum to zero.

         Reference:
         R.J. Plemmons, Regular splittings and the discrete Neumann
         problem, Numer. Math., 25 (1976), pp. 153-161.
    """
    try:
        m, n = n.shape
    except AttributeError:
        m = int(np.sqrt(n))
        if m**2 != n:
            raise ValueError('N must be a perfect square.')
        n = m

    t = tridiag(m, -1, 2, -1).todense()
    t[0, 1] = -2
    t[m - 1, m - 2] = -2

    a = np.kron(t, np.eye(n)) + np.kron(np.eye(n), t)

    return a, t



import numpy as np
from rogues.matrices.jordbloc import jordbloc


def forsythe(n, alpha=None, lambduh=0):
    """
    forsythe  forsythe matrix - a perturbed jordan block.
          forsythe(n, alpha, lambda) is the n-by-n matrix equal to
          jordbloc(n, lambda) except it has an alpha in the [n - 1, 0] position
          It has the characteristic polynomial
                  det(a-t*eye) = (lambda-t)^n - (-1)^n alpha.
          alpha defaults to sqrt(eps) and lambduh to 0.
    """
    if alpha is None:
        alpha = np.sqrt(np.finfo(float).eps)

    a = jordbloc(n, lambduh)
    a[n - 1, 0] = alpha

    return a



import numpy as np
import numpy.linalg as nl
import pylab as plt
from rogues.utils import rq, cpltaxes


class Higham(Exception):
    pass


def fv(b, nk=1, thmax=16, do_plot=True):
    """
    FV     Field of values (or numerical range).
       fv(b, nk, thmax, do_plot) evaluates and plots the field of values of the
       NK largest leading principal submatrices of b, using THMAX
       equally spaced angles in the complex plane.
       The defaults are NK = 1 and THMAX = 16.
       (For a `publication quality' picture, set THMAX higher, say 32.)
       The eigenvalues of b are displayed as `x'.
       Alternative usage: f, e = fv(a, nk, thmax, False) suppresses the
       plot and returns the field of values plot data in F, with A's
       eigenvalues in E.   Note that norm(f, inf) approximates the
       numerical radius,
                 max {abs(z): z is in the field of values of A}.

       Theory:
       Field of values fv(a) = set of all Rayleigh quotients. fv(a) is a
       convex set containing the eigenvalues of A.  When A is normal fv(a) is
       the convex hull of the eigenvalues of A (but not vice versa).
               z = x.T * a * x / np.dot(x,x),
               z.T = x.T * a.T * x / np.dot(x,x)
               => z.real = x.T * h * x / np.dot(x,x),   h = (a + a.T)/2
       so      min(eig(h)) <= z.real <= max(eig(h)),
       with equality for x = corresponding eigenvectors of h.  For these x,
       rq(a,x) is on the boundary of fv(a).

       Based on an original routine by A. Ruhe.

       References:
       R. A. Horn and C. R. Johnson, Topics in Matrix Analysis, Cambridge
            University Press, 1991; sec. 1.5.
       A. S. Householder, The Theory of Matrices in Numerical Analysis,
            Blaisdell, New York, 1964; sec. 3.3.
       C. R. Johnson, Numerical determination of the field of values of a
            general complex matrix, SIAM J. Numer. Anal., 15 (1978),
            pp. 595-602.

       Note added in porting... changed the last parameter's name.  Now it
       will suppress ploting when set to False.
    """

    thmax = thmax - 1     # Because code below uses thmax + 1 angles.

    if len(b.shape) != 2:
        raise Higham('Matrix must be two dimensional.')
    else:
        n, p = b.shape
        if n != p:
            raise Higham('Matrix must be square.')

    f = None
    z = np.zeros(2*thmax + 1, dtype=np.complex128)
    e, v = nl.eig(b)

    # Filter out cases where B is Hermitian or skew-Hermitian, for efficiency.
    if (b == b.T).all():
        f = np.hstack((np.min(e), np.max(e)))

    elif (b == -b.T).all():
        e = e.imag
        f = np.hstack((np.min(e), np.max(e)))
        e = 1j * e
        f = 1j * f

    else:
        for m in range(nk):
            ns = n - m
            a = b[0:ns, 0:ns]

            for i in range(thmax):
                th = i / float(thmax) * np.pi
                # Rotate A through angle th.
                ath = np.exp(1j * th) * a
                # Hermitian part of rotated A.
                h = 0.5 * (ath + ath.T)
                d, x = nl.eig(h)
                k = np.argsort(d.real)
                # rq's of a corr. to eigenvalues of h
                z[i] = rq(a, x[:, k[0]])
                # with smallest/largest real part.
                z[i + thmax] = rq(a, x[:, k[ns-1]])

            if f is None:
                f = z
            else:
                f = np.vstack((f, z))

        # Next line ensures boundary is `joined up' (needed for orthogonal
        # matrices).
        if len(f.shape) > 1:
            f = np.vstack((f, f[0, :]))
        else:
            f = np.vstack((f, f))

    if thmax == 0:
        f = e

    if do_plot:
        ax = cpltaxes(f, plt)
        # Plot the field of values
        plt.plot(f.real, f.imag, 'o')
        plt.axis(ax)
        plt.axis('equal')
        # Plot the eigenvalues too.
        plt.plot(e.real, e.imag, 'x')

    plt.show()
    return f, e



def skewpart(a):
    """
    skewpart  Skew-symmetric (skew-Hermitian) part.
          skewpart(a) is the skew-symmetric (skew-Hermitian) part of A,
          (A - A')/2.
          It is the nearest skew-symmetric (skew-Hermitian) matrix to A in
          both the 2- and the Frobenius norms.
    """
    s = (a - a.T) / 2.

    return s



import numpy as np


class Higham(Exception):
    pass


def treshape(x, unit=0, row_wise=False):
    """
    treshape  reshape vector to or from (unit) triangular matrix.
          treshape(x) returns a square upper triangular matrix whose
          elements are taken columnwise from the matrix x.
          If row_wise = True, then the matrix elements are taken
          row wise from the matrix x.
          treshape(x,1) returns a UNIT upper triangular matrix, and
          the 1's should not be specified in X.
          An error results if X does not have a number of elements of the form
          n*(n+1)/2 (or n less than this in the unit triangular case).
          x = treshape(r,2) is the inverse operation to r = treshape(x).
          x = treshape(r,3) is the inverse operation to r = treshape(x,1).
    """
    try:
        p, q = x.shape
    except ValueError:
        p, = x.shape
        q = 1

    if unit < 2:   # Convert vector x to upper triangular R.
        m = p * q
        n = int(np.around((-1 + np.sqrt(1 + 8 * m)) / 2.))
        if n * (n + 1) // 2 != m:
            raise Higham('Matrix must have a "triangular" '
                         'number of elements.')

        if unit == 1:
            n = n + 1

        x = x.ravel()
        t = unit * np.eye(n)

        i = 0
        if row_wise:
            for j in range(n - unit):
                t[j, j + unit:n] = x[i:i + n - unit - j]
                i = i + n - unit - j

        else:
            for j in range(unit, n):
                t[0:j - unit + 1, j] = x[i:i + j - unit + 1]
                i = i + j - unit + 1

    elif unit >= 2:   # Convert upper triangular R to vector x.
        t = x
        if p != q:
            raise Higham('Must pass square matrix')

        unit = unit - 2
        n = p * (p + 1) // 2 - unit * p
        x = np.zeros(n)
        i = 0
        if row_wise:
            for j in range(p - unit):
                x[i:i + p - unit - j] = t[j, j + unit:p]
                i = i + p - unit - j

        else:
            for j in range(unit, p):
                x[i:i + j - unit + 1] = t[0:j - unit + 1, j]
                i = i + j - unit + 1

        t = x

    return t



import numpy as np


def pei(n, alpha=1):
    """
    PEI    Pei matrix.
       pei(n, alpha), where alpha is a scalar, is the symmetric matrix
       alpha*eye(n) + ones((n,n)).
       If alpha is omitted then alpha = 1 is used.
       The matrix is singular for ALPHA = 0, -N.

       Reference:
       M.L. Pei, A test matrix for inversion procedures,
       Comm. ACM, 5 (1962), p. 508.
    """
    p = alpha * np.eye(n) + 1.

    return p



import numpy as np
from rogues.matrices import cauchy


def dingdong(n):
    """
    dingdong  dingdong matrix - a symmetric Hankel matrix.
          a = dingdong(n) is the symmetric n-by-n Hankel matrix with
                         a(i,j) = 0.5/(n-i-j+1.5).
          the eigenvalues of a cluster around pi/2 and -pi/2.

          Invented by F.N. Ris.

          Reference:
          J.C. Nash, Compact Numerical Methods for Computers: Linear
          Algebra and Function Minimisation, second edition, Adam Hilger,
          Bristol, 1990 (Appendix 1).
    """
    p = -2 * np.arange(1, n + 1) + (n + 1.5)
    a = cauchy(p)

    return a



import numpy as np


def lauchli(n, mu=None):
    """
    LAUCHLI   Lauchli matrix - rectangular.
          lauchli(n, mu) is the (n+1)-by-n matrix vstack(ones(1), mu*eye(n))
          it is a well-known example in least squares and other problems
          that indicates the dangers of forming a'*a.
          mu defaults to sqrt(eps).

          Reference:
          P. Lauchli, Jordan-Elimination und Ausgleichung nach
          kleinsten Quadraten, Numer. Math, 3 (1961), pp. 226-240.
    """
    if mu is None:
        mu = np.sqrt(np.finfo(float).eps)

    a = np.vstack((np.ones(n), mu * np.eye(n)))

    return a



import numpy as np
from rogues.matrices import cauchy


def parter(n):
    """
    PARTER    Parter matrix - a Toeplitz matrix with singular values near PI.
          PARTER(N) is the matrix with (i,j) element 1/(i-j+0.5).
          It is a Cauchy matrix and a Toeplitz matrix.

          At the Second SIAM Conference on Linear Algebra, Raleigh, N.C.,
          1985, Cleve Moler noted that most of the singular values of
          PARTER(N) are very close to PI.  An explanation of the phenomenon
          was given by Parter; see also the paper by Tyrtyshnikov.

          References:
          The MathWorks Newsletter, Volume 1, Issue 1, March 1986, page 2.
          S.V. Parter, On the distribution of the singular values of Toeplitz
               matrices, Linear Algebra and Appl., 80 (1986), pp. 115-130.
          E.E. Tyrtyshnikov, Cauchy-Toeplitz matrices and some applications,
               Linear Algebra and Appl., 149 (1991), pp. 1-18.
    """
    a = cauchy(np.arange(1, n + 1) + 0.5, -np.arange(1, n + 1))

    return a



import numpy as np
import rogues


def dramadah(n, k=1):
    """
    dramadah  a (0,1) matrix whose inverse has large integer entries.
          An anti-hadamard matrix a is a matrix with elements 0 or 1 for
          which mu(a) := norm(inv(a),'fro') is maximal.
          a = dramadah(n, k) is an n-by-n (0,1) matrix for which mu(a) is
          relatively large, although not necessarily maximal.
          Available types (the default is k = 1):
          k = 1: a is toeplitz, with abs(det(a)) = 1, and mu(a) > c(1.75)^n,
                 where c is a constant.
          k = 2: a is upper triangular and toeplitz.
          the inverses of both types have integer entries.

          Another interesting (0,1) matrix:
          k = 3: A has maximal determinant among (0,1) lower Hessenberg
          matrices: det(A) = the n'th Fibonacci number.  A is Toeplitz.
          The eigenvalues have an interesting distribution in the complex
          plane.

          References:
          R.L. Graham and N.J.A. Sloane, Anti-Hadamard matrices,
             Linear Algebra and Appl., 62 (1984), pp. 113-137.
          L. Ching, The maximum determinant of an nxn lower Hessenberg
             (0,1) matrix, Linear Algebra and Appl., 183 (1993), pp. 147-153.
    """

    if k == 1:
        # Toeplitz
        c = np.ones(n)
        for i in range(1, n, 4):
            m = min(1, n - i)
            c[i:i + m + 1] = 0

        r = np.zeros(n)
        r[0:4] = np.array([1, 1, 0, 1])
        if n < 4:
            r = r[0:n]

        a = rogues.toeplitz(c, r)

    elif k == 2:
        # Upper triangular and Toeplitz
        c = np.zeros(n)
        c[0] = 1
        r = np.ones(n)
        r[2::2] = 0

        a = rogues.toeplitz(c, r)

    elif k == 3:
        # Lower Hessenberg.
        c = np.ones(n)
        c[1::2] = 0
        a = rogues.toeplitz(c, np.hstack((np.array([1, 1]), np.zeros(n - 2))))

    return a



import numpy as np
from rogues.matrices import hilb


def wilk(n):
    """
    wilk   Various specific matrices devised/discussed by Wilkinson.
       a, b = wilk(n) is the matrix or system of order N.
       N = 3: upper triangular system Ux=b illustrating inaccurate solution.
       N = 4: lower triangular system Lx=b, ill-conditioned.
       N = 5: HILB(6)(1:5,2:6)*1.8144.  Symmetric positive definite.
       N = 21: W21+, tridiagonal.   Eigenvalue problem.

       References:
       J.H. Wilkinson, Error analysis of direct methods of matrix inversion,
          J. Assoc. Comput. Mach., 8 (1961),  pp. 281-330.
       J.H. Wilkinson, Rounding Errors in Algebraic Processes, Notes on Applied
          Science No. 32, Her Majesty's Stationery Office, London, 1963.
       J.H. Wilkinson, The Algebraic Eigenvalue Problem, Oxford University
          Press, 1965.
    """
    b = []
    if n == 3:
        # Wilkinson (1961) p.323.
        a = [[1e-10,   .9,  -.4],
             [0,     .9,  -.4],
             [0,     0,  1e-10]]
        b = [0,      0,    1]

    elif n == 4:
        # Wilkinson (1963) p.105.
        a = [[0.9143e-4, 0, 0, 0],
             [0.8762, 0.7156e-4, 0, 0],
             [0.7943, 0.8143, 0.9504e-4, 0],
             [0.8017, 0.6123, 0.7165, 0.7123e-4]]
        b = [0.6524, 0.3127, 0.4186, 0.7853]

    elif n == 5:
        # Wilkinson (1965), p.234.
        a = hilb(6, 6)
        # drop off the last row and the first column
        a = a[0:5, 1:6] * 1.8144
        # return zero array for b
        b = np.zeros(5)

    elif n == 21:
        # Taken from gallery.m.  Wilkinson (1965), p.308.
        E = np.diag(np.ones(n - 1), 1)
        m = (n - 1) / 2
        a = np.diag(np.abs(np.arange(-m, m + 1))) + E + E.T
        # return zero array for b
        b = np.zeros(21)

    else:
        raise ValueError("Sorry, that value of N is not available.")

    return np.array(a), np.array(b)



import numpy as np
import numpy.random as nrnd


class Higham(Exception):
    pass


def sparsify(a, p=0.25):
    """
    SPARSIFY  Randomly set matrix elements to zero.
          S = SPARSIFY(A, P) is A with elements randomly set to zero
          (S = S' if A is square and A = A', i.e. symmetry is preserved).
          Each element has probability P of being zeroed.
          Thus on average 100*P percent of the elements of A will be zeroed.
          Default: P = 0.25.

          Note added in porting: by inspection only, it appears the the m*lab
          version may have a bug where it always returns zeros on the diagonal
          for a symmetric matrix... can anyone confirm?
     """

    if p < 0 or p > 1:
        raise Higham('Second parameter must be between 0 and 1 inclusive.')

    m, n = a.shape

    if (a == a.T).all():
        # Preserve symmetry
        d = np.choose(nrnd.rand(m) > p, (np.zeros(m), np.diag(a)))
        a = np.triu(a, 1) * (nrnd.rand(m, n) > p)
        a = a + a.T
        a = a + np.diag(d)
    else:
        # Unsymmetric case
        a = np.choose(nrnd.rand(m, n) > p, (np.zeros((m, n)), a))

    return a



import numpy as np
from rogues.utils import cpltaxes


class Higham(Exception):
    pass


def ps(a, m=None, tol=1e-3, rl=0, marksize=0):
    """
    PS     Dot plot of a pseudospectrum.
       ps(a, m, tol, rl) plots an approximation to a pseudospectrum
       of the square matrix A, using M random perturbations of size TOL.
       M defaults to a SIZE(A)-dependent value and TOL to 1E-3.
       RL defines the type of perturbation:
         RL =  0 (default): absolute complex perturbations of 2-norm TOL.
         RL =  1:           absolute real perturbations of 2-norm TOL.
         RL = -1:           componentwise real perturbations of size TOL.
       The eigenvalues of A are plotted as crosses `x'.
       PS(A, M, TOL, RL, MARKSIZE) uses the specified marker size instead
       of a size that depends on the figure size, the matrix order, and M.
       If MARKSIZE < 0, the plot is suppressed and the plot data is returned
       as an output argument.
       PS(A, 0) plots just the eigenvalues of A.

       For a given TOL, the pseudospectrum of A is the set of
       pseudo-eigenvalues of A, that is, the set
       { e : e is an eigenvalue of A+E, for some E with NORM(E) <= TOL }.

       References:
       L. N. Trefethen, Computation of pseudospectra, Acta Numerica,
          8:247-295, 1999.
       L. N. Trefethen, Spectra and pseudospectra, in The Graduate
          Student's Guide to Numerical Analysis '98, M. Ainsworth,
          J. Levesley, and M. Marletta, eds., Springer-Verlag, Berlin,
          1999, pp. 217-250.
    """

    try:
        plt = __import__('pylab')
    except ImportError:
        raise Higham('Error importing pylab')

    if np.diff(a.shape)[0] != 0:
        raise Higham('Matrix must be square.')

    n = max(a.shape)

    if m is None:
        m = int(5 * max(1, np.around(25 * np.exp(-0.047 * n))))

    if m == 0:
        e, v = np.linalg.eig(a)
        ax = cpltaxes(e, plt)
        plt.plot(e.real, e.imag, 'rx')
        plt.axis(ax)
        plt.axis('equal')
        plt.show()
        return

    # If we don't create x with dtype=np.complex128, then
    # it defaults to float and the imaginary part of the eigenvalues
    # disappear (or we get an error, which is better) when we later
    # assign into x.
    x = np.empty(m * n, dtype=np.complex128)

    for j in range(m):
        # Componentwise.
        if rl == -1:
            # Uniform random numbers on [-1, 1].
            da = -np.ones(n) + 2 * np.random.rand(n, n)
            da = tol * (a * da)
        else:
            if rl == 0:
                # Complex absolute.
                da = np.random.randn(n, n) + 1j * np.random.randn(n, n)
            else:
                # Real absolute.
                da = np.random.randn(n, n)

            da = (tol / np.linalg.norm(da)) * da

        x[j * n:(j + 1) * n], v = np.linalg.eig(a + da)

    if marksize >= 0:
        ax = cpltaxes(x, plt)
        h = plt.plot(x.real, x.imag, '.')
        plt.axis(ax)
        plt.axis('equal')

        if marksize == 0:
            # Apparently the 'units' property is not supported in
            # matplotlib and the postion property returns values
            # that have been normalized between 0 and 1 and are
            # a Bbox instance (a [2,2] array not a [4] array).  Not
            # clear how to proceed in picking the marker size for
            # the pseudo eigenvalues so we punt an put in a default
            marksize = 5

        e, v = np.linalg.eig(a)
        plt.plot(e.real, e.imag, 'rx')
        plt.setp(h, 'markersize', marksize)
        plt.show()

    else:
        y = x
        return y

    return None



import numpy as np
from rogues.matrices import hilb


def lotkin(n):
    """
    lotkin  lotkin matrix.
        a = lotkin(n) is the Hilbert matrix with its first row altered to
        all ones.  A is unsymmetric, ill-conditioned, and has many negative
        eigenvalues of small magnitude.
        The inverse has integer entries and is known explicitly.

        Reference:
        M. Lotkin, A set of test matrices, MTAC, 9 (1955), pp. 153-161.
    """
    a = hilb(n)
    a[0, :] = np.ones(n)

    return a



import numpy as np


def kahan(n, theta=1.2, pert=25):
    """
    kahan  kahan matrix - upper trapezoidal.
       kahan(n, theta) is an upper trapezoidal matrix
       that has some interesting properties regarding estimation of
       condition and rank.
       The matrix is n-by-n unless n is a 2-tuple, in which case it
       is n[0]-by-n[1].
       The parameter theta defaults to 1.2.
       The useful range of theta is 0 < theta < pi.

       To ensure that the QR factorization with column pivoting does not
       interchange columns in the presence of rounding errors, the diagonal
       is perturbed by pert*eps*np.diag( arange(n,0,-1)).
       The default is pert = 25, which ensures no interchanges for kahan(n)
       up to at least n = 90 in IEEE arithmetic.
       kahan(n, theta, pert) uses the given value of pert.

       The inverse of kahan(n, theta) is known explicitly: see
       Higham (1987, p. 588), for example.
       The diagonal perturbation was suggested by Christian Bischof.

       References:
       W. Kahan, Numerical linear algebra, Canadian Math. Bulletin,
          9 (1966), pp. 757-801.
       N.J. Higham, A survey of condition number estimation for
          triangular matrices, SIAM Review, 29 (1987), pp. 575-596.
    """

    try:
        r, n = n              # Parameter n specifies dimension: r-by-n.
    except TypeError:
        r = n

    s = np.sin(theta)
    c = np.cos(theta)

    u = np.eye(n) - c * np.triu(np.ones((n, n)), 1)
    z = np.diag(s ** np.arange(0, n))
    eps = np.finfo(float).eps
    u = z @ u + pert * eps * np.diag(np.arange(n, 0, -1))

    if r > n:
        # Extend to an r-by-n matrix.
        u = np.vstack((u, np.zeros(((r - n), n))))
    elif r < n:
        # Reduce to an r-by-n matrix.
        u = u[:r, :]

    return u



import numpy as np


def jordbloc(n, lambduh=1):
    """
    jordbloc  jordan block.
          jordbloc(n, lambda) is the n-by-n jordan block with eigenvalue
          lambduh.  lambduh = 1 is the default. (Recall that lambda is a
          reserved word in Python)
    """

    j = lambduh * np.eye(n) + np.diag(np.ones(n - 1), 1)

    return j



import numpy as np
import rogues
import scipy.linalg as sl


def condex(n, k=4, theta=100):
    """
    CONDEX   `Counterexamples' to matrix condition number estimators.
         CONDEX(N, K, THETA) is a `counterexample' matrix to a condition
         estimator.  It has order N and scalar parameter THETA (default 100).
         If N is not equal to the `natural' size of the matrix then
         the matrix is padded out with an identity matrix to order N.
         The matrix, its natural size, and the estimator to which it applies
         are specified by K (default K = 4) as follows:
             K = 1:   4-by-4,     LINPACK (RCOND)
             K = 2:   3-by-3,     LINPACK (RCOND)
             K = 3:   arbitrary,  LINPACK (RCOND) (independent of THETA)
             K = 4:   N >= 4,     SONEST (Higham 1988)
         (Note that in practice the K = 4 matrix is not usually a
          counterexample because of the rounding errors in forming it.)

         References:
         A.K. Cline and R.K. Rew, A set of counter-examples to three
            condition number estimators, SIAM J. Sci. Stat. Comput.,
            4 (1983), pp. 602-611.
         N.J. Higham, FORTRAN codes for estimating the one-norm of a real or
            complex matrix, with applications to condition estimation
            (Algorithm 674), ACM Trans. Math. Soft., 14 (1988), pp. 381-396.
    """

    if k == 1:    # Cline and Rew (1983), Example B.

        a = np.array([[1, -1, -2 * theta, 0],
                      [0, 1, theta, -theta],
                      [0, 1, 1 + theta, -(theta + 1)],
                      [0, 0, 0, theta]])

    elif k == 2:   # Cline and Rew (1983), Example C.

        a = np.array([[1, 1 - 2 / theta ** 2, -2],
                      [0, 1 / theta, -1 / theta],
                      [0, 0, 1]])

    elif k == 3:    # Cline and Rew (1983), Example D.

        a = rogues.triw(n, -1).T
        a[-1, -1] = -1

    elif k == 4:    # Higham (1988), p. 390.

        x = np.ones((n, 3))             # First col is e
        x[1:n, 1] = np.zeros(n - 1)     # Second col is e(1)

        # Third col is special vector b in SONEST
        x[:, 2] = ((-1) ** np.arange(n)) * (1 + np.arange(n) / (n - 1))

        # Q*Q' is now the orthogonal projector onto span(e(1),e,b)).
        q = sl.orth(x)
        p = np.eye(n) - np.asmatrix(q) * np.asmatrix(q.T)
        a = np.eye(n) + theta * p

    # Pad out with identity as necessary.
    m, m = a.shape
    if m < n:
        for i in range(n - 1, m, -1):
            a[i, i] = 1

    return a



import numpy as np


def chebvand(m, p=None):
    """
    CHEBVAND  Vandermonde-like matrix for the Chebyshev polynomials.
          c = chebvand(p), where P is a vector, produces the (primal)
          Chebyshev Vandermonde matrix based on the points P,
          i.e., c[i,j] = t_{i-1}(p[j]), where T_{i-1} is the Chebyshev
          polynomial of degree i-1.
          CHEBVAND(M,P) is a rectangular version of CHEBVAND(P) with M rows.
          Special case: If P is a scalar then P equally spaced points on
                        [0,1] are used.

           Reference:
           N.J. Higham, Stability analysis of algorithms for solving confluent
           Vandermonde-like systems, SIAM J. Matrix Anal. Appl., 11 (1990),
           pp. 23-41.
    """
    if p is None:
        nargin = 1
        p = m
    else:
        nargin = 2

    try:
        n = np.max(p.shape)
    except AttributeError:
        n = p
        p = np.linspace(0., 1., n)

    if nargin == 1:
        m = n

    c = np.ones((m, n))
    if m == 1:
        return c

    c[1, :] = p

    #  Use Chebyshev polynomial recurrence.
    for i in range(2, m):
        c[i, :] = 2. * p * c[i - 1, :] - c[i - 2, :]

    return c



import numpy as np


def comp(a, k=0):
    """
    COMP    Comparison matrices.
        comp(a) is diag(b) - tril(b,-1) - triu(b,1), where b = abs(a).
        comp(a, 1) is a with each diagonal element replaced by its
        absolute value, and each off-diagonal element replaced by minus
        the absolute value of the largest element in absolute value in
        its row.  however, if a is triangular comp(a, 1) is too.
        comp(a, 0) is the same as comp(a).
        comp(a) is often denoted by m(a) in the literature.

        Reference (e.g.):
        N.J. Higham, A survey of condition number estimation for
        triangular matrices, SIAM Review, 29 (1987), pp. 575-596.
    """
    m, n = a.shape
    p = min(m, n)

    if k == 0:

        # This code uses less temporary storage than the `high level'
        # definition above. (well, maybe... not clear that this is so
        # in numpy as opposed to m*lab)
        c = -abs(a)
        for j in range(p):
            c[j, j] = np.abs(a[j, j])

    elif k == 1:

        c = a.T
        for j in range(p):
            c[k, k] = 0

        mx = np.empty(p)
        for j in range(p):
            mx[j] = max(abs(c[j, :]))

        c = -np.outer(mx * np.ones(n))
        for j in range(p):
            c[j, j] = abs(a[j, j])

        if (a == np.tril(a)).all():
            c = np.tril(c)
        if (a == np.triu(a)).all():
            c = np.triu(c)

    return c



import numpy as np


def vand(p, m=0):
    """
    VAND   Vandermonde matrix.
       v = vand(p), where p is a vector, produces the (primal)
       Vandermonde matrix based on the points p, i.e. v[i,j] = p[j]**(i-1)
       vand(p, m) is a rectangular version of vand(p) with m rows
       Special case: If P is a scalar then P equally spaced points on [0,1]
                     are used.

       Reference:
       N. J. Higham, Accuracy and Stability of Numerical Algorithms,
       Second edition, Society for Industrial and Applied Mathematics,
       Philadelphia, PA, 2002; chap. 22

       Notes added in porting:
       You should probably use np.vander instead of this function, but
       then the interface is different and you get the row reversed
       transpose of the matrix ( ie a.T[::-1,:] ) returned by vand

       WARNING... changed the function signiture from the Higham m*lab version
       by reversing the order of the arguments.
    """
    try:
        n = len(p)
    except TypeError:
        #  Handle scalar p.
        n = p
        p = np.linspace(0, 1, n)

    if m == 0:
        m = n

    v = np.ones((m, n))
    for i in range(1, m):
        v[i, :] = p * v[i - 1, :]

    return v



import numpy as np
from rogues.utils import toeplitz


def prolate(n, w=0.25):
    """
    PROLATE   Prolate matrix - symmetric, ill-conditioned Toeplitz matrix.
          A = PROLATE(N, W) is the N-by-N prolate matrix with parameter W.
          It is a symmetric Toeplitz matrix.
          If 0 < W < 0.5 then
             - A is positive definite
             - the eigenvalues of A are distinct, lie in (0, 1), and
               tend to cluster around 0 and 1.
          W defaults to 0.25.

          Reference:
          J.M. Varah. The Prolate matrix. Linear Algebra and Appl.,
          187:269--278, 1993.
    """
    a = np.zeros(n)
    a[0] = 2 * w
    a[1:n] = np.sin(2 * np.pi * w * np.arange(1, n)) / (np.pi *
                                                        np.arange(1, n))
    t = toeplitz(a)

    return t



import numpy as np


def redheff(n):
    """
    redheff    a (0,1) matrix of redheffer associated with the
               riemann hypothesis.
           a = redheff(n) is an n-by-n matrix of 0s and 1s defined by
               a[i,j] = 1 if j = 0 or if (i+1) divides (j+1),
               a[i,j] = 0 otherwise.
           It has n - floor(log2(n)) - 1 eigenvalues equal to 1,
           a real eigenvalue (the spectral radius) approximately sqrt(n),
           a negative eigenvalue approximately -sqrt(n),
           and the remaining eigenvalues are provably ``small''.
           barrett and jarvis (1992) conjecture that
             ``the small eigenvalues all lie inside the unit circle
               abs(z) = 1'',
           and a proof of this conjecture, together with a proof that some
           eigenvalue tends to zero as n tends to infinity, would yield
           a new proof of the prime number theorem.
           The Riemann hypothesis is true if and only if
           det(a) = o( n^(1/2+epsilon) ) for every epsilon > 0
                                             (`!' denotes factorial).
           see also riemann().

           Reference:
           W.W. Barrett and T.J. Jarvis,
           Spectral Properties of a Matrix of Redheffer,
           Linear Algebra and Appl., 162 (1992), pp. 673-683.
    """
    i = np.outer(np.arange(1, n + 1), np.ones(n))
    a = np.where(np.remainder(i.T, i) == 0, 1, 0)
    a[:, 0] = np.ones(n)

    return a



import numpy as np


def gfpp(t, c=1.0):
    """
    GFPP   Matrix giving maximal growth factor for Gaussian elim. with pivoting
       GFPP(T) is a matrix of order N for which Gaussian elimination
       with partial pivoting yields a growth factor 2^(N-1).
       T is an arbitrary nonsingular upper triangular matrix of order N-1.
       GFPP(T, C) sets all the multipliers to C  (0 <= C <= 1)
       and gives growth factor (1+C)^(N-1).
       GFPP(N, C) (a special case) is the same as GFPP(EYE(N-1), C) and
       generates the well-known example of Wilkinson.

       Reference:
       N.J. Higham and D.J. Higham, Large growth factors in
       Gaussian elimination with pivoting, SIAM J. Matrix Analysis and
       Appl., 10 (1989), pp. 155-164.
    """

    try:
        m, = t.shape
        # t must be an array rather than an int
        n = m + 1
        if np.linalg.norm(t - np.triu(t), 1) | (np.diag(t) == 0.0).any():
            raise ValueError('First argument must be a nonsingular upper'
                             ' triangular matrix.')
    except AttributeError:
        # Handle the special case T = scalar
        n = t
        m = n - 1
        t = np.eye(n - 1)

    if c < 0. or c > 1.:
        raise Higham('Second parameter must be a scalar between 0 and '
                     '1 inclusive.')

    d = 1. + c
    l = np.eye(n) - c * np.tril(np.ones((n, n)), -1)

    uu = np.hstack((t, (d ** np.arange(0, n - 1)).reshape(n - 1, 1)))
    ul = np.zeros((1, m + 1))
    ul[0, m] = d ** (n - 1)
    u = np.vstack((uu, ul))
    a = l @ u
    theta = np.abs(a).max()
    a[:, n - 1] = (theta / np.linalg.norm(a[:, n - 1], np.inf)) * a[:, n - 1]

    return np.asarray(a)



import numpy as np
import scipy.sparse as sparse


class Higham(Exception):
    pass


def wathen(nx, ny, k=0):
    """
    WATHEN  Wathen matrix - a finite element matrix (sparse, random entries).
        a = wathen(nx, ny) is a sparse random n-by-n finite element matrix
        where n = 3*nx*ny + 2*nx + 2*ny + 1.
        A is precisely the `consistent mass matrix' for a regular NX-by-NY
        grid of 8-node (serendipity) elements in 2 space dimensions.
        A is symmetric positive definite for any (positive) values of
        the `density', rho(nx,ny), which is chosen randomly in this routine.
        in particular, if d = diag(diag(a)), then
              0.25 <= eig(inv(d)*a) <= 4.5
        for any positive integers nx and ny and any densities rho(nx,ny).
        this diagonally scaled matrix is returned by wathen(nx,ny,1).
        For k = 1 wathen returns the diagonally scaled matrix.

        Reference:
        A.J. Wathen, Realistic eigenvalue bounds for the Galerkin
        mass matrix, IMA J. Numer. Anal., 7 (1987), pp. 449-457.
    """
    e1 = np.array([[6, -6, 2, -8],
                   [-6, 32, -6, 20],
                   [2, -6, 6, -6],
                   [-8, 20, -6, 32]])

    e2 = np.array([[3, -8, 2, -6],
                   [-8, 16, -8, 20],
                   [2, -8, 3, -8],
                   [-6, 20, -8, 16]])

    ea = np.hstack((e1, e2))
    eb = np.hstack((e2.T, e1))
    e = np.vstack((ea, eb)) / 45.
    n = 3 * nx * ny + 2 * nx + 2 * ny + 1
    a = sparse.lil_matrix((n, n))

    rho = np.random.randint(1, 100, (nx, ny))
    nn = np.zeros(8)

    for j in range(ny):
        for i in range(nx):
            nn[0] = 3 * j * nx + 3 * nx + 2 * i + 2 * j + 4
            nn[1] = nn[0] - 1
            nn[2] = nn[1] - 1
            nn[3] = (3 * j + 2) * nx + 2 * j + i + 1
            nn[4] = 3 * j * nx + 2 * i + 2 * j
            nn[5] = nn[4] + 1
            nn[6] = nn[5] + 1
            nn[7] = nn[3] + 1
            nn = nn.astype(int)
            
            em = e * rho[i, j]

            for krow in range(8):
                for kcol in range(8):
                    a[nn[krow], nn[kcol]] = a[nn[krow], nn[kcol]] + \
                                            em[krow, kcol]

    if k == 1:
        raise Higham("k = 1 option not supported... ignoring")

    return a



import numpy as np


class Higham(Exception):
    pass


def pdtoep(n, m=None, w=None, theta=None):
    """
    PDTOEP   Symmetric positive definite Toeplitz matrix.
         PDTOEP(N, M, W, THETA) is an N-by-N symmetric positive (semi-)
         definite (SPD) Toeplitz matrix, comprised of the sum of M rank 2
         (or, for certain THETA, rank 1) SPD Toeplitz matrices.
         Specifically,
                 T = W(1)*T(THETA(1)) + ... + W(M)*T(THETA(M)),
         where T(THETA(k)) has (i,j) element COS(2*PI*THETA(k)*(i-j)).
         Defaults: M = N, W = RAND(M,1), THETA = RAND(M,1).

         Reference:
         G. Cybenko and C.F. Van Loan, Computing the minimum eigenvalue of
         a symmetric positive definite Toeplitz matrix, SIAM J. Sci. Stat.
         Comput., 7 (1986), pp. 123-131.
    """
    if m is None:
        m = n

    if w is None:
        w = np.random.rand(m)

    if theta is None:
        theta = np.random.rand(m)

    try:
        if np.max(w.shape) != m or np.max(theta.shape) != m:
            raise Higham('Arguments w and theta must be vectors of length M.')
    except:
        raise Higham('Arguments w and theta must be vectors of length M.')

    t = np.zeros(n)
    e = 2 * np.pi * (np.outer(np.arange(1, n + 1), np.ones(n)) -
                     np.ones(n) * np.arange(1, n + 1))

    for i in range(m):
        t = t + w[i] * np.cos(theta[i] * e)

    return t



import numpy as np


def riemann(n):
    """
    RIEMANN    A matrix associated with the Riemann hypothesis.
           A = RIEMANN(N) is an N-by-N matrix for which the
           Riemann hypothesis is true if and only if
           DET(A) = O( N! N^(-1/2+epsilon) ) for every epsilon > 0
                                             (`!' denotes factorial).
           A = B(2:N+1, 2:N+1), where
           B(i,j) = i-1 if i divides j and -1 otherwise.
           Properties include, with M = N+1:
              Each eigenvalue E(i) satisfies ABS(E(i)) <= M - 1/M.
              i <= E(i) <= i+1 with at most M-SQRT(M) exceptions.
              All integers in the interval (M/3, M/2] are eigenvalues.

           See also REDHEFF.

           Reference:
           F. Roesler, Riemann's hypothesis as an eigenvalue problem,
           Linear Algebra and Appl., 81 (1986), pp. 153-198.
    """
    n = n + 1
    i = np.outer(np.arange(2, n + 1), np.ones(n - 1))
    j = i.T
    a = np.where(np.remainder(j, i) == 0, i, -1)

    return a



"""
The "rogues" module is a reimplementation of Prof. N. Higham's test matrices into
Python, Numpy, and Scipy.  They were generally ported by using the
iPython shell.  They were developed from Version 3 (1995) of the test
matrix package from code downloaded from Higham's web site.  Also,
the earlier TOMS 694 version was used as reference in some cases.
That code was download from netlib.  Both of these packages were
accessed in February of 2009.

Also included are a small number of routines from Prof. Higham's matrixcomp
library as well as several required functions that had no implementations.

Some of the issues in porting to numpy from m*lab
    * numpy arrays have zero based array indexing while m*lab has one's
      based array indexing
    * numpy / python has indexing that does not include the upper end of
      the specified range, e.g. a[0:n]  is  a[0], a[1], ..., a[n-1].  This
      is different from m*lab
    * Of course, it is much easier to handle default values on the input
      parameters in Python
    * Element by element operation is the default in numpy.  To get the
      matrix behavior, the array must be converted to matrices.  Also,
      when dealing with arrays, we do not need to use the dot notation of
      m*lab  (ie x./y).  Also numpy has the a concept called broadcasting
      so that we can write and expression such as 1/x  which, if x is a
      array becomes  [[1/x[0,0], 1/x[0,1], ... rather than ones(n,n)./x
    * Some of the numpy functions take tuples for the shapes of arrays
      (notably zeros, ones, etc) while others do not (random.randn())
    * The m*lab routines that take matrix size arguments generally assume
      that a single dimension, say n, means the matrix is square, say n by n.
      This means that when you want a vector, you have to give the function
      _two_ arguments ie say zeros(n,1) or ones(1,n) etc. In numpy, one
      dimension is the default and we use zeros(n) etc.  When we need a
      two dimensional array we use zeros((m,n))
      
Comments and references were mostly preserved in the functions.  They were
slightly updated to reflect the changes necessary in Python

The inluded matrix generation functions are:

   cauchy
   chebspec
   chebvand
   chow
   clement
   comp
   compan
   condex
   cycol
   dingdong
   dorr
   dramadah
   fiedler
   forsythe
   frank
   gearm
   gfpp
   grcar
   hadamard
   hankel
   hanowa
   hilb
   invhess
   invol
   ipjfact
   jordbloc
   kahan
   kms
   krylov
   lauchli
   lehmer
   lesp
   lotkin
   minij
   moler
   neumann
   ohess
   parter
   pascal
   pdtoep
   pei
   pentoep
   poisson
   prolate
   qmult
   rando
   randsvd
   redheff
   riemann
   rogues
   smoke
   test
   triw
   wathen
   wilk
   wilkinson

Some of generally useful matrix utility functions:

   augment
   bandred
   cgs
   cond
   condeig
   cpltaxes
   dual
   ge
   hankel
   house
   mdsmax
   mgs
   pow2
   ps
   repmat
   rq
   skewpart
   sparsify
   sub
   symmpart
   toeplitz
   treshape
   tridiag
   vand
   vecperm

   More information is available on any of these functions by typing
   "help <funcname>"
   
Don MacMillen 1 August 2018
"""
name = "rogues"

__version__ = "0.5.0"
   
from rogues.matrices import *
from rogues.utils import *




import numpy as np


def compan(p):
    """
    COMPAN  Companion matrix.
        compan(p) is a companion matrix.  There are three cases.
        If p is a scalar then compan(p) is the p-by-p matrix compan(1:p+1).
        If p is an (n+1)-vector, compan(p) is the n-by-n companion matrix
           whose first row is -p(2:n+1)/p(1).
        If p is a square matrix, compan(p) is the companion matrix
           of the characteristic polynomial of p, computed as
           compan(poly(p)).

        References:
        J.H. Wilkinson, The Algebraic Eigenvalue Problem,
           Oxford University Press, 1965, p. 12.
        G.H. Golub and C.F. Van Loan, Matrix Computations, second edition,
           Johns Hopkins University Press, Baltimore, Maryland, 1989,
           sec 7.4.6.
        C. Kenney and A.J. Laub, Controllability and stability radii for
          companion form systems, Math. Control Signals Systems, 1 (1988),
          pp. 239-256. (Gives explicit formulas for the singular values of
          COMPAN(P).)
    """
    try:
        n, m = p.shape
        if n == m and n > 1:
            # Matrix argument
            a = compan(np.poly(p))
            return a
        else:
            # Hmm, the commented stmt matches the logic in the m*lab code but
            # it would seem to be on a false path in the control logic,
            # ie compan shouldn't be defined for a non square matrix
            # so here we error out when that happens.
            # n = max(n,m)
            raise ValueError("Input matrix 'a' must be square "
                             "with dimension > 1")

    except ValueError:
        n, = p.shape

    except AttributeError:
        n = p + 1
        p = np.arange(1, n + 1)

    # Construct matrix of order n - 1
    if n == 2:
        a = 1
    else:
        a = np.diag(np.ones(n - 2), -1)
        a[0, :] = -p[1:n] / p[0]

    return a



import numpy as np
import scipy.sparse as sparse


def pentoep(n, a=1, b=-10, c=0, d=10, e=1):
    """
    PENTOEP   Pentadiagonal Toeplitz matrix (sparse).
          p = pentoep(n, a, b, c, d, e) is the n-by-n pentadiagonal
          Toeplitz matrix with diagonals composed of the numbers
          a =: p[3,1], b =: p[2,1], c =: p[1,1], d =: p[1,2], e =: p[1,3].
          default: (a,b,c,d,e) = (1,-10,0,10,1) (a matrix of rutishauser).
                    This matrix has eigenvalues lying approximately on
                    the line segment 2*cos(2*t) + 20*i*sin(t).

          Interesting plots are
          ps(full(pentoep(32,0,1,0,0,1/4)))  - `triangle'
          ps(full(pentoep(32,0,1/2,0,0,1)))  - `propeller'
          ps(full(pentoep(32,0,1/2,1,1,1)))  - `fish'

          References:
          R.M. Beam and R.F. Warming, The asymptotic spectra of
             banded Toeplitz and quasi-Toeplitz matrices, SIAM J. Sci.
             Comput. 14 (4), 1993, pp. 971-1006.
          H. Rutishauser, On test matrices, Programmation en Mathematiques
             Numeriques, Editions Centre Nat. Recherche Sci., Paris, 165,
             1966, pp. 349-365.
    """
    o1 = np.ones(n)
    data = np.vstack((a * o1, b * o1, c * o1, d * o1, e * o1))
    diags = np.arange(-2, 3)
    p = sparse.spdiags(data, diags, n, n)

    return p.todense()



import numpy as np
import numpy.linalg as nl
from rogues.matrices import qmult
from rogues.utils import bandred


class Higham(Exception):
    pass


def randsvd(n, kappa=None, mode=3, kl=None, ku=None):
    """
    RANDSVD  Random matrix with pre-assigned singular values.
      randsvd(n, kappa, mode, kl, ku) is a (banded) random matrix of order n
      with cond(a) = kappa and singular values from the distribution mode.
      n may be a 2-tuple, in which case the matrix is n[0]-by-n[1].
      Available types:
             mode = 1:   one large singular value,
             mode = 2:   one small singular value,
             mode = 3:   geometrically distributed singular values,
             mode = 4:   arithmetically distributed singular values,
             mode = 5:   random singular values with unif. dist. logarithm.
      If omitted, mode defaults to 3, and kappa defaults to sqrt(1/eps).
      If mode < 0 then the effect is as for abs(mode) except that in the
      original matrix of singular values the order of the diagonal entries
      is reversed: small to large instead of large to small.
      KL and KU are the lower and upper bandwidths respectively; if they
      are omitted a full matrix is produced.
      If only KL is present, KU defaults to KL.
      Special case: if KAPPA < 0 then a random full symmetric positive
                    definite matrix is produced with cond(a) = -kappa and
                    eigenvalues distributed according to mode.
                    kl and ku, if present, are ignored.

      Reference:
      N.J. Higham, Accuracy and Stability of Numerical Algorithms,
         Society for Industrial and Applied Mathematics, Philadelphia, PA,
         USA, 2002; sec. 28.3.

      This routine is similar to the more comprehensive Fortran routine xLATMS
      in the following reference:
      J.W. Demmel and A. McKenney, A test matrix generation suite,
      LAPACK Working Note #9, Courant Institute of Mathematical Sciences,
      New York, 1989.
    """
    # Parameter n specifies dimension: m-by-n.
    try:
        m, n = n
        p = min(m, n)
    except TypeError:
        m = n
        p = n

    if kappa is None:
        kappa = np.sqrt(1 / np.finfo(float).eps)

    if kl is None:
        kl = n - 1          # Full matrix.

    if ku is None:
        ku = kl             # Same upper and lower bandwidths.

    if np.abs(kappa) < 1:
        raise Higham('Condition number must be at least 1!')

    is_posdef = False
    if kappa < 0:
        is_posdef = True
        kappa = -kappa

    if p == 1:              # Handle case where a is a vector, not a matrix
        a = np.random.randn(max(m, n))
        a = a / nl.norm(a)
        return a

    j = np.abs(mode)

    # Set up vector sigma of singular values.
    if j == 3:
        factor = kappa ** (-1 / (p - 1))
        sigma = factor ** np.arange(0, p)

    elif j == 4:
        sigma = np.ones(p) - np.arange(p) / (p - 1) * (1 - 1 / kappa)

    elif j == 5:
        # In this case cond(a) <= kappa
        sigma = np.exp(-np.random.rand(size=p) * np.log(kappa))

    elif j == 2:
        sigma = np.ones(p)
        sigma[p - 1] = 1 / kappa

    elif j == 1:
        sigma = np.ones(p) / kappa
        sigma[0] = 1

    # Convert to diagonal matrix of singular values.
    if mode < 0:
        sigma = sigma[::-1]

    sigma = np.diag(sigma)

    if is_posdef:               # Handle special case.
        q = qmult(p)
        a = q.T @ sigma @ q
        a = (a + a.T) / 2.      # Ensure matrix is symmetric.
        return a

    # Expand matrix, if necessary
    if m > n:
        sigma = np.vstack((sigma, np.zeros((m - n, n))))
    elif m < n:
        sigma = np.hstack((sigma, np.zeros((m, n - m))))

    if kl == 0 and ku == 0:  # Diagonal matrix requested - nothing more to do.
        a = sigma
        return a

    # A = U*sigma*V, where U, V are random orthogonal matrices from the
    # Haar distribution.
    a = qmult(sigma.T)
    a = qmult(a.T)

    if kl < n - 1 or ku < n - 1:   # Bandwidth reduction.
        a = bandred(a, kl, ku)

    return a



import numpy as np


def pow2(x, y=None):
    """
    Raise 2 to the power of x[i] for the vector x.  If two vectors
    are supplied, return    x[i] * (2 ** y[i])
    Note that no error checking is done in this example.
    """
    if y is None:
        z = (2. * np.ones(len(x))) ** x
    else:
        z = x * ((2. * np.ones(len(y))) ** y)

    return z



import numpy as np


class Higham(Exception):
    pass


def wilkinson(n):
    """
    wilkinson array of size n where n must be odd.
    This is what some others call a Wilkinson array for arbitrary n. Note
    that Higham only uses this definition for n = 21
    """
    if not n % 2:
        raise Higham("n must be odd")

    m = int(n / 2)
    y = np.diag(np.abs(np.arange(-m, m + 1)))
    x1 = np.diag(np.ones(2 * m), 1)
    x2 = np.diag(np.ones(2 * m), -1)
    return y + x1 + x2
